/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/
#include <glib.h>

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <merc.h>
#include <recycle.h>
#include <olc.h>
#include <interp.h>
#include <fight.h>
#include <tables.h>

/*
 * Give an obj to a char.
 */
void obj_to_char( OBJ_DATA *obj, CHAR_DATA *ch )
{
	if (obj == NULL) return;
	obj->next_content	 = ch->carrying;
	ch->carrying	 = obj;
	obj->carried_by	 = ch;
	obj->in_room	 = NULL;
	obj->in_obj		 = NULL;
	ch->carry_number	+= 1;
	ch->carry_weight	+= get_obj_weight( obj );
}

bool check_artifact( CHAR_DATA *ch )
{
	OBJ_DATA *obj;
	OBJ_DATA *in_obj;
	bool found;
	GSList *tmp = object_list;
	
	found = FALSE;
	while ( tmp != NULL )
	{
		obj = (OBJ_DATA*)tmp->data;
		if ( !IS_SET(obj->quest, QUEST_ARTIFACT) ) 
        {
        	tmp = g_slist_next(tmp);
		    continue;
        }
        if (obj->carried_by != ch) 
        {
        	tmp = g_slist_next(tmp);
		    continue;
        }
        
		found = TRUE;
		
		for ( in_obj = obj; in_obj->in_obj != NULL; in_obj = in_obj->in_obj )
			;
		tmp = g_slist_next(tmp);
	}
	g_slist_free(tmp);
	return found;
}


/*
 * Take an obj from its character.
 */
void obj_from_char( OBJ_DATA *obj )
{
	CHAR_DATA *ch;
	
	if (obj == NULL) return;
	if ( ( ch = obj->carried_by ) == NULL )
	{
		bug( "Obj_from_char: null ch.", 0 );
		return;
	}
	
	if ( obj->wear_loc != WEAR_NONE )
		unequip_char( ch, obj );
	
	if ( ch->carrying == obj )
	{
		ch->carrying = obj->next_content;
	}
	else
	{
		OBJ_DATA *prev;
		
		for ( prev = ch->carrying; prev != NULL; prev = prev->next_content )
		{
			if ( prev->next_content == obj )
			{
				prev->next_content = obj->next_content;
				break;
			}
		}
		
		if ( prev == NULL )
			bug( "Obj_from_char: obj not in list.", 0 );
	}
	
	obj->carried_by	 = NULL;
	obj->next_content	 = NULL;
	ch->carry_number	-= 1;
	ch->carry_weight	-= get_obj_weight( obj );
	return;
}
/*
 * Find a piece of eq on a character.
 */
OBJ_DATA *get_eq_char( CHAR_DATA *ch, int iWear )
{
	OBJ_DATA *obj;
	
	for ( obj = ch->carrying; obj != NULL; obj = obj->next_content )
	{
		if ( obj->wear_loc == iWear )
			return obj;
	}
	
	return NULL;
}



/*
 * Equip a char with an obj.
 */
void equip_char( CHAR_DATA *ch, OBJ_DATA *obj, int iWear )
{
	CHAR_DATA   *chch;
	AFFECT_DATA *paf;
	int sn;
	
	if ( get_eq_char( ch, iWear ) != NULL )
	{
		bug( "Equip_char: already equipped (%d).", iWear );
		return;
	}
	
	
	if ( ( IS_OBJ_STAT(obj, ITEM_ANTI_EVIL)    && IS_EVIL(ch)    )
		||   ( IS_OBJ_STAT(obj, ITEM_ANTI_GOOD)    && IS_GOOD(ch)    )
		||   ( IS_OBJ_STAT(obj, ITEM_ANTI_NEUTRAL) && IS_NEUTRAL(ch) ) )
	{
	/*
	 * Thanks to Morgenes for the bug fix here!
	 */
		act( "You are zapped by $p and drop it.", ch, obj, NULL, TO_CHAR );
		act( "$n is zapped by $p and drops it.",  ch, obj, NULL, TO_ROOM );
		obj_from_char( obj );
		obj_to_room( obj, ch->in_room );
		do_autosave( ch, "" );
		return;
	}
	
	
	if ((IS_NPC(ch) || (!IS_CLASS(ch, CLASS_DEMON))) &&
		IS_SET(obj->spectype, SITEM_DEMONIC))
	{
		act( "You are zapped by $p and drop it.", ch, obj, NULL, TO_CHAR );
		act( "$n is zapped by $p and drops it.",  ch, obj, NULL, TO_ROOM );
		obj_from_char( obj );
		obj_to_room( obj, ch->in_room );
		do_autosave( ch, "" );
		return;
	}
	
	if ((IS_NPC(ch) || !IS_IMMORTAL(ch)) && IS_SET(obj->quest,QUEST_RELIC)
		&&  !IS_SET(obj->spectype,SITEM_DEMONIC)
		&&  !IS_SET(obj->spectype,SITEM_HIGHLANDER)
		&&  obj->chobj == NULL)
	{
		act( "You are zapped by $p and drop it.", ch, obj, NULL, TO_CHAR );
		act( "$n is zapped by $p and drops it.",  ch, obj, NULL, TO_ROOM );
		obj_from_char( obj );
		obj_to_room( obj, ch->in_room );
		do_autosave( ch, "" );
		return;
	}
/*
 * Mercpoint - commented out to enable wearing more than one artifact.
 * Damcap is not modified by more than 500 on wearing more than one
 * artifact due to the fact that the damcap modifier is based on whether
 * a player is affected by ITEMA_ARTIFACT and as this can only be either
 * on or off it only affects damcap once :)
 * Therefore this enables a player to wear all the artifacts they have
 * aquired (call it a reward for having got them) as because we have
 * changed the code such that artifacts can no longer be put in bags,
 * it now follows that a player with more than one artifact who is decapped
 * will have his corpse looted for all of those artifacts.
 * Hence he might as well can the benefit for having them.
 *
 * It miht be an idea to add a check such that if the player is still
 * wearing an artifact when they remove one that the ITEMA_ARTIFACT bit
 * isn't removed. - Merc (4/10/96)
 *
    if (IS_SET(obj->quest, QUEST_ARTIFACT) && IS_ITEMAFF(ch, ITEMA_ARTIFACT))
    {
	act( "You are zapped by $p and drop it.", ch, obj, NULL, TO_CHAR );
	act( "$n is zapped by $p and drops it.",  ch, obj, NULL, TO_ROOM );
	obj_from_char( obj );
	obj_to_room( obj, ch->in_room );
	do_autosave( ch, "" );
	return;
    }
 */
	
	if ((IS_NPC(ch) || !IS_IMMORTAL( ch )) && IS_SET(obj->spectype, SITEM_DROWWEAPON))
	{
		act( "You are zapped by $p and drop it.", ch, obj, NULL, TO_CHAR );
		act( "$n is zapped by $p and drops it.",  ch, obj, NULL, TO_ROOM );
		obj_from_char( obj );
		obj_to_room( obj, ch->in_room );
		do_autosave( ch, "" );
		return;
	}
	
	if ((IS_NPC(ch) || !IS_CLASS(ch, CLASS_HIGHLANDER)) 
		&& IS_SET(obj->spectype, SITEM_HIGHLANDER))
	{
		act( "You are zapped by $p and drop it.", ch, obj, NULL, TO_CHAR );
		act( "$n is zapped by $p and drops it.",  ch, obj, NULL, TO_ROOM );
		obj_from_char( obj );
		obj_to_room( obj, ch->in_room );
		do_autosave( ch, "" );
		return;
	}
	
	if (obj->item_type == ITEM_WWF_CONT)
    {
        if (IS_SET(obj->value[0], AFF_BLIND) && !IS_AFFECTED(ch,AFF_BLIND))
            SET_BIT(ch->affected_by, AFF_BLIND);
        if (IS_SET(obj->value[0], AFF_FLYING) && !IS_AFFECTED(ch,AFF_FLYING))
            SET_BIT(ch->affected_by, AFF_FLYING);
        if (IS_SET(obj->value[0], AFF_INVISIBLE) && !IS_AFFECTED(ch,AFF_INVISIBLE))
            SET_BIT(ch->affected_by, AFF_INVISIBLE);
        if (IS_SET(obj->value[0], AFF_PASS_DOOR) && !IS_AFFECTED(ch,AFF_PASS_DOOR))
            SET_BIT(ch->affected_by, AFF_PASS_DOOR);
        if (IS_SET(obj->value[0], AFF_PROTECT) && !IS_AFFECTED(ch,AFF_PROTECT))
            SET_BIT(ch->affected_by, AFF_PROTECT);
        if (IS_SET(obj->value[0], AFF_SANCTUARY) && !IS_AFFECTED(ch,AFF_SANCTUARY))
            SET_BIT(ch->affected_by, AFF_SANCTUARY);
        if (IS_SET(obj->value[0], AFF_SNEAK) && !IS_AFFECTED(ch,AFF_SNEAK))
            SET_BIT(ch->affected_by, AFF_SNEAK);

	if (!IS_IMMFLAW(ch,IMM_FLAW_LIGHTNING))
	{
		if (IS_SET(obj->value[1], ITEMA_SHOCKSHIELD) && !IS_ITEMAFF(ch,ITEMA_SHOCKSHIELD))
			SET_BIT(ch->itemaffect, ITEMA_SHOCKSHIELD);
	}

	if (!IS_IMMFLAW(ch,IMM_FLAW_HEAT))
	{
        	if (IS_SET(obj->value[1], ITEMA_FIRESHIELD) && !IS_ITEMAFF(ch,ITEMA_FIRESHIELD))
            		SET_BIT(ch->itemaffect, ITEMA_FIRESHIELD);
	}

	if (!IS_IMMFLAW(ch,IMM_FLAW_COLD))
	{
		if (IS_SET(obj->value[1], ITEMA_ICESHIELD) && !IS_ITEMAFF(ch,ITEMA_ICESHIELD))
			SET_BIT(ch->itemaffect, ITEMA_ICESHIELD);
	}

	if (!IS_IMMFLAW(ch,IMM_FLAW_ACID))
	{
        	if (IS_SET(obj->value[1], ITEMA_ACIDSHIELD) && !IS_ITEMAFF(ch,ITEMA_ACIDSHIELD))
            		SET_BIT(ch->itemaffect, ITEMA_ACIDSHIELD);
	}

        if (IS_SET(obj->value[1], ITEMA_DBPASS) && !IS_ITEMAFF(ch,ITEMA_DBPASS))
            SET_BIT(ch->itemaffect, ITEMA_DBPASS);
        if (IS_SET(obj->value[1], ITEMA_CHAOSSHIELD) && !IS_ITEMAFF(ch,ITEMA_CHAOSSHIELD))
            SET_BIT(ch->itemaffect, ITEMA_CHAOSSHIELD);
        if (IS_SET(obj->value[1], ITEMA_REGENERATE) && !IS_ITEMAFF(ch,ITEMA_REGENERATE))
            SET_BIT(ch->itemaffect, ITEMA_REGENERATE);
        if (IS_SET(obj->value[1], ITEMA_SPEED) && !IS_ITEMAFF(ch,ITEMA_SPEED))
            SET_BIT(ch->itemaffect, ITEMA_SPEED);
        if (IS_SET(obj->value[1], ITEMA_VORPAL) && !IS_ITEMAFF(ch,ITEMA_VORPAL))
            SET_BIT(ch->itemaffect, ITEMA_VORPAL);
        if (IS_SET(obj->value[1], ITEMA_PEACE) && !IS_ITEMAFF(ch,ITEMA_PEACE))
            SET_BIT(ch->itemaffect, ITEMA_PEACE);
        if (IS_SET(obj->value[1], ITEMA_REFLECT) && !IS_ITEMAFF(ch,ITEMA_REFLECT))
            SET_BIT(ch->itemaffect, ITEMA_REFLECT);
        if (IS_SET(obj->value[1], ITEMA_RESISTANCE) && !IS_ITEMAFF(ch,ITEMA_RESISTANCE))
            SET_BIT(ch->itemaffect, ITEMA_RESISTANCE);
        if (IS_SET(obj->value[1], ITEMA_VISION) && !IS_ITEMAFF(ch,ITEMA_VISION))
            SET_BIT(ch->itemaffect, ITEMA_VISION);
        if (IS_SET(obj->value[1], ITEMA_STALKER) && !IS_ITEMAFF(ch,ITEMA_STALKER))
            SET_BIT(ch->itemaffect, ITEMA_STALKER);
        if (IS_SET(obj->value[1], ITEMA_RAGER) && !IS_ITEMAFF(ch,ITEMA_RAGER))
            SET_BIT(ch->itemaffect, ITEMA_RAGER);
        if (IS_SET(obj->value[1], ITEMA_VANISH) && !IS_ITEMAFF(ch,ITEMA_VANISH))
            SET_BIT(ch->itemaffect, ITEMA_VANISH);
        if (IS_SET(obj->value[1], ITEMA_FORTITUDE) && !IS_ITEMAFF(ch,ITEMA_FORTITUDE))
            SET_BIT(ch->itemaffect, ITEMA_FORTITUDE);
    }

	if ( iWear == WEAR_SCABBARD_L ||
		iWear == WEAR_SCABBARD_R ) 
		{
			obj->wear_loc = iWear;
			return;
		}
	
	if ( IS_NPC(ch) || !IS_CLASS(ch, CLASS_HIGHLANDER) ) 
		ch->armor -= apply_ac( obj, iWear );
	
	obj->wear_loc = iWear;
	
	for ( paf = obj->pIndexData->affected; paf != NULL; paf = paf->next )
		affect_modify( ch, paf, TRUE );
	
	for ( paf = obj->affected; paf != NULL; paf = paf->next )
	{
		if ( paf->bitvector != 1 && obj->value[0] != 5)
			affect_modify( ch, paf, TRUE );
	}
	
	if ( obj->item_type == ITEM_LIGHT
		&&   obj->value[2] != 0
		&&   ch->in_room != NULL )
		++ch->in_room->light;
	
	if (!IS_NPC(ch))
	{
		if ( (chch = get_char_world(ch,ch->name->str)) == NULL ) return;
		if ( chch->desc != ch->desc) return;
	}
	
	
	if (obj->chpoweron != NULL 
		&& str_cmp(obj->chpoweron->str,"(null)")
		&& !IS_SET(obj->spectype, SITEM_TELEPORTER)
		&& !IS_SET(obj->spectype, SITEM_TRANSPORTER) )
	{
		kavitem(obj->chpoweron->str,ch,obj,NULL,TO_CHAR);
		if (IS_SET(obj->spectype, SITEM_ACTION))
			kavitem(obj->chpoweron->str,ch,obj,NULL,TO_ROOM);
	}
	if (obj->victpoweron != NULL 
		&& str_cmp(obj->victpoweron->str,"(null)")
		&& !IS_SET(obj->spectype, SITEM_ACTION)
		&& !IS_SET(obj->spectype, SITEM_TELEPORTER)
		&& !IS_SET(obj->spectype, SITEM_TRANSPORTER) )
		kavitem(obj->victpoweron->str,ch,obj,NULL,TO_ROOM);
	
	if ( obj->wear_loc == WEAR_NONE )
		return;
	if (   ((obj->item_type == ITEM_ARMOR ) && (obj->value[3] >= 1   ))
		|| ((obj->item_type == ITEM_WEAPON) && (obj->value[0] >= 1000))
		|| IS_SET(obj->spectype, SITEM_SILVER)
		|| IS_SET(obj->spectype, SITEM_DEMONIC)
		|| IS_SET(obj->quest, QUEST_ARTIFACT) )
	{
	/* It would be so much easier if weapons had 5 values *sigh*.  
	 * Oh well, I'll just have to use v0 for two.  KaVir.
	 */
		if (obj->item_type == ITEM_ARMOR)
			sn = obj->value[3];
		else
			sn = obj->value[0] / 1000;
		
		
		if      ((sn ==  4) && (IS_AFFECTED(ch, AFF_BLIND)))        return;
		else if ((sn == 39) && (IS_AFFECTED(ch, AFF_FLYING)))       return;
		else if ((sn == 46) && (IS_AFFECTED(ch, AFF_INVISIBLE)))    return;
		else if ((sn == 52) && (IS_AFFECTED(ch, AFF_PASS_DOOR)))    return;
		else if ((sn == 54) && (IS_AFFECTED(ch, AFF_PROTECT)))      return;
		else if ((sn == 57) && (IS_AFFECTED(ch, AFF_SANCTUARY)))    return;
		else if ((sn == 3 ) && (IS_AFFECTED(ch, AFF_FLYING)))       return;
		else if ((sn == 5 ) && (IS_AFFECTED(ch, AFF_INVISIBLE)))    return;
		else if ((sn == 6 ) && (IS_AFFECTED(ch, AFF_PASS_DOOR)))    return;
		else if ((sn == 7 ) && (IS_AFFECTED(ch, AFF_PROTECT)))      return;
		else if ((sn == 8 ) && (IS_AFFECTED(ch, AFF_SANCTUARY)))    return;
		else if ((sn == 9 ) && (IS_AFFECTED(ch, AFF_SNEAK)))        return;
		else if ((sn == 10) && (IS_ITEMAFF(ch, ITEMA_SHOCKSHIELD))) return;
		else if ((sn == 11) && (IS_ITEMAFF(ch, ITEMA_FIRESHIELD)))  return;
		else if ((sn == 12) && (IS_ITEMAFF(ch, ITEMA_ICESHIELD)))   return;
		else if ((sn == 13) && (IS_ITEMAFF(ch, ITEMA_ACIDSHIELD)))  return;
		else if ((sn == 14) && (IS_ITEMAFF(ch, ITEMA_DBPASS)))      return;
		else if ((sn == 15) && (IS_ITEMAFF(ch, ITEMA_CHAOSSHIELD))) return;
		else if ((sn == 16) && (IS_ITEMAFF(ch, ITEMA_REGENERATE)))  return;
		else if ((sn == 17) && (IS_ITEMAFF(ch, ITEMA_SPEED)))       return;
		else if ((sn == 18) && (IS_ITEMAFF(ch, ITEMA_VORPAL)))      return;
		else if ((sn == 19) && (IS_ITEMAFF(ch, ITEMA_PEACE)))       return;
		else if ((sn == 20) && (IS_ITEMAFF(ch, ITEMA_REFLECT)))     return;
		else if ((sn == 21) && (IS_ITEMAFF(ch, ITEMA_RESISTANCE)))  return;
		else if ((sn == 22) && (IS_ITEMAFF(ch, ITEMA_VISION)))      return;
		else if ((sn == 23) && (IS_ITEMAFF(ch, ITEMA_STALKER)))     return;
		else if ((sn == 24) && (IS_ITEMAFF(ch, ITEMA_VANISH)))      return;
		else if ((sn == 25) && (IS_ITEMAFF(ch, ITEMA_RAGER)))       return;
		else if ((sn == 66) && (IS_ITEMAFF(ch, ITEMA_FORTITUDE)))   return;
		if (sn == 4)
		{
			SET_BIT(ch->affected_by, AFF_BLIND);
			send_to_char( "...You cannot see a thing!\n\r", ch );
			act("...$n seems to be blinded!",ch,NULL,NULL,TO_ROOM);
		}
		else if (sn == 39 || sn == 3)
		{
			SET_BIT(ch->affected_by, AFF_FLYING);
			send_to_char( "...Your feet rise off the ground.\n\r", ch );
			act("...$n's feet rise off the ground.",ch,NULL,NULL,TO_ROOM);
		}
		else if (sn == 46 || sn == 5)
		{
			SET_BIT(ch->affected_by, AFF_INVISIBLE);
			send_to_char( "...You fade out of existance.\n\r", ch );
			act("...$n fades out of existance.",ch,NULL,NULL,TO_ROOM);
		}
		else if (sn == 52 || sn == 6)
		{
			SET_BIT(ch->affected_by, AFF_PASS_DOOR);
			send_to_char( "...You turn translucent.\n\r", ch );
			act("...$n turns translucent.",ch,NULL,NULL,TO_ROOM);
		}
		else if (sn == 54 || sn == 7)
		{
			SET_BIT(ch->affected_by, AFF_PROTECT);
			send_to_char( "...You are surrounded by a divine aura.\n\r", ch );
			act("...$n is surrounded by a divine aura.",ch,NULL,NULL,TO_ROOM);
		}
		else if (sn == 57 || sn == 8)
		{
			SET_BIT(ch->affected_by, AFF_SANCTUARY);
			send_to_char( "...You are surrounded by a white aura.\n\r", ch );
			act("...$n is surrounded by a white aura.",ch,NULL,NULL,TO_ROOM);
		}
		else if (sn == 9)
		{
			SET_BIT(ch->affected_by, AFF_SNEAK);
			send_to_char( "...Your footsteps stop making any sound.\n\r", ch );
			act("...$n's footsteps stop making any sound.",ch,NULL,NULL,TO_ROOM);
		}
		else if (sn == 10)
		{
			SET_BIT(ch->itemaffect, ITEMA_SHOCKSHIELD);
			send_to_char( "...You are surrounded by a crackling shield of lightning.\n\r", ch );
			act("...$n is surrounded by a crackling shield of lightning.",ch,NULL,NULL,TO_ROOM);
		}
		else if (sn == 11)
		{
			SET_BIT(ch->itemaffect, ITEMA_FIRESHIELD);
			send_to_char( "...You are surrounded by a burning shield of flames.\n\r", ch );
			act("...$n is surrounded by a burning shield of flames.",ch,NULL,NULL,TO_ROOM);
		}
		else if (sn == 12)
		{
			SET_BIT(ch->itemaffect, ITEMA_ICESHIELD);
			send_to_char( "...You are surrounded by a shimmering shield of ice.\n\r", ch );
			act("...$n is surrounded by a shimmering shield of ice.",ch,NULL,NULL,TO_ROOM);
		}
		else if (sn == 13)
		{
			SET_BIT(ch->itemaffect, ITEMA_ACIDSHIELD);
			send_to_char( "...You are surrounded by a bubbling shield of acid.\n\r", ch );
			act("...$n is surrounded by a bubbling shield of acid.",ch,NULL,NULL,TO_ROOM);
		}
		else if (sn == 14)
		{
			SET_BIT(ch->itemaffect, ITEMA_DBPASS);
			send_to_char( "...You are now safe from the DarkBlade clan guardians.\n\r", ch );
		}
		else if (sn == 15)
		{
			SET_BIT(ch->itemaffect, ITEMA_CHAOSSHIELD);
			send_to_char( "...You are surrounded by a swirling shield of chaos.\n\r", ch );
			act("...$n is surrounded by a swirling shield of chaos.",ch,NULL,NULL,TO_ROOM);
		}
		else if (sn == 16)
			SET_BIT(ch->itemaffect, ITEMA_REGENERATE);
		else if (sn == 17)
		{
			SET_BIT(ch->itemaffect, ITEMA_SPEED);
			send_to_char( "...You start moving faster than the eye can follow.\n\r", ch );
			act("...$n starts moving faster than the eye can follow.",ch,NULL,NULL,TO_ROOM);
		}
		else if (sn == 18)
			SET_BIT(ch->itemaffect, ITEMA_VORPAL);
		else if (sn == 19)
			SET_BIT(ch->itemaffect, ITEMA_PEACE);
		else if (sn == 20)
		{
			SET_BIT(ch->itemaffect, ITEMA_REFLECT);
			send_to_char( "...You are surrounded by flickering shield of darkness.\n\r", ch );
			act("...$n is surrounded by a flickering shield of darkness.",ch,NULL,NULL,TO_ROOM);
		}
		else if (sn == 21)
			SET_BIT(ch->itemaffect, ITEMA_RESISTANCE);
		else if (sn == 22)
		{
			SET_BIT(ch->itemaffect, ITEMA_VISION);
			send_to_char( "...Your eyes begin to glow bright white.\n\r", ch );
			act("...$n's eyes begin to glow bright white.",ch,NULL,NULL,TO_ROOM);
		}
		else if (sn == 23)
			SET_BIT(ch->itemaffect, ITEMA_STALKER);
		else if (sn == 24)
		{
			SET_BIT(ch->itemaffect, ITEMA_VANISH);
			send_to_char( "...You blend into the shadows.\n\r", ch );
			act("...$n gradually fades into the shadows.",ch,NULL,NULL,TO_ROOM);
		}
		else if (sn == 25 && !IS_NPC(ch))
		{
			SET_BIT(ch->itemaffect, ITEMA_RAGER);
			if (IS_CLASS(ch, CLASS_WEREWOLF) && ch->pcdata->stats[UNI_RAGE] < 100)
			{
				ch->pcdata->stats[UNI_RAGE] = 300;
				do_werewolf(ch,"");
				//update_damcap(ch,ch);
			}
			else if (IS_CLASS(ch, CLASS_WEREWOLF))
				ch->pcdata->stats[UNI_RAGE] = 300;
			else if (IS_CLASS(ch, CLASS_VAMPIRE))
				ch->pcdata->stats[UNI_RAGE] = 25;
		}
		else if (sn == 66)
		{
			SET_BIT(ch->itemaffect, ITEMA_FORTITUDE);
			send_to_char( "...You twitch suddenly.\n\r", ch )
			;
			act("...$n twitches suddenly.",ch,NULL,NULL,TO_ROOM);
		}
		if (IS_SET(obj->quest, QUEST_ARTIFACT))
			SET_BIT(ch->itemaffect, ITEMA_ARTIFACT);
		if (IS_SET(obj->spectype, SITEM_SILVER) && obj->wear_loc == WEAR_WIELD)
			SET_BIT(ch->itemaffect, ITEMA_RIGHT_SILVER);
		if (IS_SET(obj->spectype, SITEM_SILVER) && obj->wear_loc == WEAR_HOLD)
			SET_BIT(ch->itemaffect, ITEMA_LEFT_SILVER);
	}
	return;
}

/*
 * Unequip a char with an obj.
 */
void unequip_char( CHAR_DATA *ch, OBJ_DATA *obj )
{
	CHAR_DATA   *chch;
	AFFECT_DATA *paf;
	int sn;
	int oldpos = obj->wear_loc;
	
	if ( obj->wear_loc == WEAR_NONE )
	{
		bug( "Unequip_char: already unequipped.", 0 );
		return;
	}
	
	if ( obj->wear_loc == WEAR_SCABBARD_L ||
		obj->wear_loc == WEAR_SCABBARD_R ) {obj->wear_loc = -1;return;}
	
	if ( IS_NPC(ch) || !IS_CLASS(ch, CLASS_HIGHLANDER) ) 
		ch->armor += apply_ac(obj, obj->wear_loc);
	
    obj->wear_loc	 = -1;
	
	for ( paf = obj->pIndexData->affected; paf != NULL; paf = paf->next )
		affect_modify( ch, paf, FALSE );
	for ( paf = obj->affected; paf != NULL; paf = paf->next )
	{
		if ( paf->bitvector != 1 && obj->value[0] != 5)
			affect_modify( ch, paf, FALSE );
	}
	
	if ( obj->item_type == ITEM_LIGHT
		&&   obj->value[2] != 0
		&&   ch->in_room != NULL
		&&   ch->in_room->light > 0 )
		--ch->in_room->light;
	
	if ( (chch = get_char_world(ch,ch->name->str)) == NULL ) return;
	if ( chch->desc != ch->desc) return;
	
	if (!IS_NPC(ch) && (ch->desc != NULL && ch->desc->connected != CON_PLAYING)) return;
	
	if (obj->chpoweroff != NULL 
		&& str_cmp(obj->chpoweroff->str,"(null)")
		&& !IS_SET(obj->spectype, SITEM_TELEPORTER)
		&& !IS_SET(obj->spectype, SITEM_TRANSPORTER) )
	{
		kavitem(obj->chpoweroff->str,ch,obj,NULL,TO_CHAR);
		if (IS_SET(obj->spectype, SITEM_ACTION))
			kavitem(obj->chpoweroff->str,ch,obj,NULL,TO_ROOM);
	}
	if (obj->victpoweroff != NULL 
		&& str_cmp(obj->victpoweroff->str,"(null)")
		&& !IS_SET(obj->spectype, SITEM_ACTION)
		&& !IS_SET(obj->spectype, SITEM_TELEPORTER)
		&& !IS_SET(obj->spectype, SITEM_TRANSPORTER) )
		kavitem(obj->victpoweroff->str,ch,obj,NULL,TO_ROOM);
	
    if (obj->item_type == ITEM_WWF_CONT)
    {
        if (IS_SET(obj->value[0], AFF_BLIND) && IS_AFFECTED(ch,AFF_BLIND))
            REMOVE_BIT(ch->affected_by, AFF_BLIND);
        if (IS_SET(obj->value[0], AFF_FLYING) && IS_AFFECTED(ch,AFF_FLYING))
            REMOVE_BIT(ch->affected_by, AFF_FLYING);
        if (IS_SET(obj->value[0], AFF_INVISIBLE) && IS_AFFECTED(ch,AFF_INVISIBLE))
            REMOVE_BIT(ch->affected_by, AFF_INVISIBLE);
        if (IS_SET(obj->value[0], AFF_PASS_DOOR) && IS_AFFECTED(ch,AFF_PASS_DOOR))
            REMOVE_BIT(ch->affected_by, AFF_PASS_DOOR);
        if (IS_SET(obj->value[0], AFF_PROTECT) && IS_AFFECTED(ch,AFF_PROTECT))
            REMOVE_BIT(ch->affected_by, AFF_PROTECT);
        if (IS_SET(obj->value[0], AFF_SANCTUARY) && IS_AFFECTED(ch,AFF_SANCTUARY))
            REMOVE_BIT(ch->affected_by, AFF_SANCTUARY);
        if (IS_SET(obj->value[0], AFF_SNEAK) && IS_AFFECTED(ch,AFF_SNEAK))
            REMOVE_BIT(ch->affected_by, AFF_SNEAK);
        if (IS_SET(obj->value[1], ITEMA_SHOCKSHIELD) && !IS_ITEMAFF(ch,ITEMA_SHOCKSHIELD))
            REMOVE_BIT(ch->itemaffect, ITEMA_SHOCKSHIELD);
        if (IS_SET(obj->value[1], ITEMA_FIRESHIELD) && !IS_ITEMAFF(ch,ITEMA_FIRESHIELD))
            REMOVE_BIT(ch->itemaffect, ITEMA_FIRESHIELD);
        if (IS_SET(obj->value[1], ITEMA_ICESHIELD) && !IS_ITEMAFF(ch,ITEMA_ICESHIELD))
            REMOVE_BIT(ch->itemaffect, ITEMA_ICESHIELD);
        if (IS_SET(obj->value[1], ITEMA_ACIDSHIELD) && !IS_ITEMAFF(ch,ITEMA_ACIDSHIELD))
            REMOVE_BIT(ch->itemaffect, ITEMA_ACIDSHIELD);
        if (IS_SET(obj->value[1], ITEMA_DBPASS) && !IS_ITEMAFF(ch,ITEMA_DBPASS))
            REMOVE_BIT(ch->itemaffect, ITEMA_DBPASS);
        if (IS_SET(obj->value[1], ITEMA_CHAOSSHIELD) && !IS_ITEMAFF(ch,ITEMA_CHAOSSHIELD))
            REMOVE_BIT(ch->itemaffect, ITEMA_CHAOSSHIELD);
        if (IS_SET(obj->value[1], ITEMA_REGENERATE) && !IS_ITEMAFF(ch,ITEMA_REGENERATE))
            REMOVE_BIT(ch->itemaffect, ITEMA_REGENERATE);
        if (IS_SET(obj->value[1], ITEMA_SPEED) && !IS_ITEMAFF(ch,ITEMA_SPEED))
            REMOVE_BIT(ch->itemaffect, ITEMA_SPEED);
        if (IS_SET(obj->value[1], ITEMA_VORPAL) && !IS_ITEMAFF(ch,ITEMA_VORPAL))
            REMOVE_BIT(ch->itemaffect, ITEMA_VORPAL);
        if (IS_SET(obj->value[1], ITEMA_PEACE) && !IS_ITEMAFF(ch,ITEMA_PEACE))
            REMOVE_BIT(ch->itemaffect, ITEMA_PEACE);
        if (IS_SET(obj->value[1], ITEMA_REFLECT) && !IS_ITEMAFF(ch,ITEMA_REFLECT))
            REMOVE_BIT(ch->itemaffect, ITEMA_REFLECT);
        if (IS_SET(obj->value[1], ITEMA_RESISTANCE) && !IS_ITEMAFF(ch,ITEMA_RESISTANCE))
            REMOVE_BIT(ch->itemaffect, ITEMA_RESISTANCE);
        if (IS_SET(obj->value[1], ITEMA_VISION) && !IS_ITEMAFF(ch,ITEMA_VISION))
            REMOVE_BIT(ch->itemaffect, ITEMA_VISION);
        if (IS_SET(obj->value[1], ITEMA_STALKER) && !IS_ITEMAFF(ch,ITEMA_STALKER))
            REMOVE_BIT(ch->itemaffect, ITEMA_STALKER);
        if (IS_SET(obj->value[1], ITEMA_RAGER) && !IS_ITEMAFF(ch,ITEMA_RAGER))
            REMOVE_BIT(ch->itemaffect, ITEMA_RAGER);
        if (IS_SET(obj->value[1], ITEMA_VANISH) && !IS_ITEMAFF(ch,ITEMA_VANISH))
            REMOVE_BIT(ch->itemaffect, ITEMA_VANISH);
        if (IS_SET(obj->value[1], ITEMA_FORTITUDE) && !IS_ITEMAFF(ch,ITEMA_FORTITUDE))
            REMOVE_BIT(ch->itemaffect, ITEMA_FORTITUDE);
    }
	
	if ( ((obj->item_type == ITEM_ARMOR ) && (obj->value[3] >= 1   ))
		|| ((obj->item_type == ITEM_WEAPON) && (obj->value[0] >= 1000))
		|| IS_SET(obj->spectype, SITEM_SILVER)
		|| IS_SET(obj->spectype, SITEM_DEMONIC)
		|| IS_SET(obj->quest, QUEST_ARTIFACT) )
	{
		if (obj->item_type == ITEM_ARMOR)
			sn = obj->value[3];
		else
			sn = obj->value[0] / 1000;
		if (IS_AFFECTED(ch, AFF_BLIND) && (sn == 4)
			&& !is_affected(ch, 4))
		{
			REMOVE_BIT(ch->affected_by, AFF_BLIND);
			send_to_char( "...You can see again.\n\r", ch );
			act("...$n seems to be able to see again.",ch,NULL,NULL,TO_ROOM);
		}
		else if (IS_AFFECTED(ch, AFF_FLYING) && (sn == 39 || sn == 3)
			&& !is_affected(ch, 39))
		{
			REMOVE_BIT(ch->affected_by, AFF_FLYING);
			send_to_char( "...You slowly float to the ground.\n\r", ch );
			act("...$n slowly floats to the ground.",ch,NULL,NULL,TO_ROOM);
		}
		else if (IS_AFFECTED(ch, AFF_INVISIBLE) && (sn == 46 || sn == 5)
			&& !is_affected(ch, 46))
		{
			REMOVE_BIT(ch->affected_by, AFF_INVISIBLE);
			send_to_char( "...You fade into existance.\n\r", ch );
			act("...$n fades into existance.",ch,NULL,NULL,TO_ROOM);
		}
		else if (IS_AFFECTED(ch, AFF_PASS_DOOR) && (sn == 52 || sn == 6)
			&& !is_affected(ch, 52))
		{
			REMOVE_BIT(ch->affected_by, AFF_PASS_DOOR);
			send_to_char( "...You feel solid again.\n\r", ch );
			act("...$n is no longer translucent.",ch,NULL,NULL,TO_ROOM);
		}
		else if (IS_AFFECTED(ch, AFF_PROTECT) && (sn == 54 || sn == 7)
			&& !is_affected(ch, 54))
		{
			REMOVE_BIT(ch->affected_by, AFF_PROTECT);
			send_to_char( "...The divine aura around you fades.\n\r", ch );
			act("...The divine aura around $n fades.",ch,NULL,NULL,TO_ROOM);
		}
		else if (IS_AFFECTED(ch, AFF_SANCTUARY) && (sn == 57 || sn == 8)
			&& !is_affected(ch, 57))
		{
			REMOVE_BIT(ch->affected_by, AFF_SANCTUARY);
			send_to_char( "...The white aura around your body fades.\n\r", ch );
			act("...The white aura about $n's body fades.",ch,NULL,NULL,TO_ROOM);
		}
		else if (IS_AFFECTED(ch, AFF_SNEAK) && (sn == 9)
			&& !is_affected(ch, 80))
		{
			REMOVE_BIT(ch->affected_by, AFF_SNEAK);
			send_to_char( "...You are no longer moving so quietly.\n\r", ch );
			act("...$n is no longer moving so quietly.",ch,NULL,NULL,TO_ROOM);
		}
		else if (IS_ITEMAFF(ch, ITEMA_SHOCKSHIELD) && (sn == 10))
		{
			REMOVE_BIT(ch->itemaffect, ITEMA_SHOCKSHIELD);
			send_to_char( "...The crackling shield of lightning around you fades.\n\r", ch );
			act("...The crackling shield of lightning around $n fades.",ch,NULL,NULL,TO_ROOM);
		}
		else if (IS_ITEMAFF(ch, ITEMA_FIRESHIELD) && (sn == 11))
		{
			REMOVE_BIT(ch->itemaffect, ITEMA_FIRESHIELD);
			send_to_char( "...The burning shield of fire around you fades.\n\r", ch );
			act("...The burning shield of fire around $n fades.",ch,NULL,NULL,TO_ROOM);
		}
		else if (IS_ITEMAFF(ch, ITEMA_ICESHIELD) && (sn == 12))
		{
			REMOVE_BIT(ch->itemaffect, ITEMA_ICESHIELD);
			send_to_char( "...The shimmering shield of ice around you fades.\n\r", ch );
			act("...The shimmering shield of ice around $n fades.",ch,NULL,NULL,TO_ROOM);
		}
		else if (IS_ITEMAFF(ch, ITEMA_ACIDSHIELD) && (sn == 13))
		{
			REMOVE_BIT(ch->itemaffect, ITEMA_ACIDSHIELD);
			send_to_char( "...The bubbling shield of acid around you fades.\n\r", ch );
			act("...The bubbling shield of acid around $n fades.",ch,NULL,NULL,TO_ROOM);
		}
		else if (IS_ITEMAFF(ch, ITEMA_DBPASS) && (sn == 14))
		{
			REMOVE_BIT(ch->itemaffect, ITEMA_DBPASS);
			send_to_char( "...You are no longer safe from the DarkBlade clan guardians.\n\r", ch );
		}
		else if (IS_ITEMAFF(ch, ITEMA_CHAOSSHIELD) && (sn == 15))
		{
			REMOVE_BIT(ch->itemaffect, ITEMA_CHAOSSHIELD);
			send_to_char( "...The swirling shield of chaos around you fades.\n\r", ch );
			act("...The swirling shield of chaos around $n fades.",ch,NULL,NULL,TO_ROOM);
		}
		else if (sn == 16)
			REMOVE_BIT(ch->itemaffect, ITEMA_REGENERATE);
		else if (IS_ITEMAFF(ch, ITEMA_SPEED) && (sn == 17))
		{
			REMOVE_BIT(ch->itemaffect, ITEMA_SPEED);
			send_to_char( "...Your actions slow down to normal speed.\n\r", ch );
			act("...$n stops moving at supernatural speed.",ch,NULL,NULL,TO_ROOM);
		}
		else if (sn == 18)
			REMOVE_BIT(ch->itemaffect, ITEMA_VORPAL);
		else if (sn == 19)
			REMOVE_BIT(ch->itemaffect, ITEMA_PEACE);
		else if (IS_ITEMAFF(ch, ITEMA_REFLECT) && sn == 20)
		{
			REMOVE_BIT(ch->itemaffect, ITEMA_REFLECT);
			send_to_char( "...The flickering shield of darkness around you fades.\n\r", ch );
			act("...The flickering shield of darkness around $n fades.",ch,NULL,NULL,TO_ROOM);
		}
		else if (sn == 21)
			REMOVE_BIT(ch->itemaffect, ITEMA_RESISTANCE);
		else if (IS_ITEMAFF(ch, ITEMA_VISION) && sn == 22)
		{
			REMOVE_BIT(ch->itemaffect, ITEMA_VISION);
			send_to_char( "...Your eyes stop glowing bright white.\n\r", ch );
			act("...$n's eyes stop glowing bright white.",ch,NULL,NULL,TO_ROOM);
		}
		else if (sn == 23)
			REMOVE_BIT(ch->itemaffect, ITEMA_STALKER);
		else if (IS_ITEMAFF(ch, ITEMA_VANISH) && sn == 24)
		{
			REMOVE_BIT(ch->itemaffect, ITEMA_VANISH);
			send_to_char( "...You emerge from the shadows.\n\r", ch );
			act("...$n gradually fades out of the shadows.",ch,NULL,NULL,TO_ROOM);
		}
		else if (sn == 25 && !IS_NPC(ch) && IS_ITEMAFF(ch, ITEMA_RAGER))
		{
			REMOVE_BIT(ch->itemaffect, ITEMA_RAGER);
			if (IS_CLASS(ch, CLASS_WEREWOLF) && ch->pcdata->stats[UNI_RAGE] >= 100)
			{
				ch->pcdata->stats[UNI_RAGE] = 0;
				do_unwerewolf(ch,"");
			}
			ch->pcdata->stats[UNI_RAGE] = 0;
		}
		else if (IS_ITEMAFF(ch, ITEMA_FORTITUDE) && sn == 66)
		{
			REMOVE_BIT(ch->itemaffect, ITEMA_FORTITUDE);
			send_to_char( "...You twitch suddenly.\n\r", ch );
			act("...$n twitches suddendly.",ch,NULL,NULL,TO_ROOM);
		}
		if (IS_SET(obj->quest, QUEST_ARTIFACT)) 
			REMOVE_BIT(ch->itemaffect, ITEMA_ARTIFACT);
		if (IS_SET(obj->spectype, SITEM_SILVER) && oldpos == WEAR_WIELD)
			REMOVE_BIT(ch->itemaffect, ITEMA_RIGHT_SILVER);
		if (IS_SET(obj->spectype, SITEM_SILVER) && oldpos == WEAR_HOLD)
			REMOVE_BIT(ch->itemaffect, ITEMA_LEFT_SILVER);
	}
	return;
}



/*
 * Count occurrences of an obj in a list.
 */
int count_obj_list( OBJ_INDEX_DATA *pObjIndex, OBJ_DATA *list )
{
	OBJ_DATA *obj;
	int nMatch;
	
	nMatch = 0;
	for ( obj = list; obj != NULL; obj = obj->next_content )
	{
		if ( obj->pIndexData == pObjIndex )
			nMatch++;
	}
	
	return nMatch;
}



/*
 * Move an obj out of a room.
 */
void obj_from_room( OBJ_DATA *obj )
{
	ROOM_INDEX_DATA *in_room;
	
	if (obj == NULL) return;
	if ( ( in_room = obj->in_room ) == NULL )
	{
		bug( "obj_from_room: NULL.", 0 );
		return;
	}
	
	if ( obj == in_room->contents )
	{
		in_room->contents = obj->next_content;
	}
	else
	{
		OBJ_DATA *prev;
		
		for ( prev = in_room->contents; prev; prev = prev->next_content )
		{
			if ( prev->next_content == obj )
			{
				prev->next_content = obj->next_content;
				break;
			}
		}
		
		if ( prev == NULL )
		{
			bug( "Obj_from_room: obj not found.", 0 );
			return;
		}
	}
	
	obj->in_room      = NULL;
	obj->next_content = NULL;
	return;
}



/*
 * Move an obj into a room.
 */
void obj_to_room( OBJ_DATA *obj, ROOM_INDEX_DATA *pRoomIndex )
{
	if (pRoomIndex == NULL) return;
	if (obj == NULL) return;
	obj->next_content		= pRoomIndex->contents;
	pRoomIndex->contents	= obj;
	obj->in_room		= pRoomIndex;
	obj->carried_by		= NULL;
	obj->in_obj			= NULL;
	return;
}






/*
 * Move an object out of an object.
 */
void obj_from_obj( OBJ_DATA *obj )
{
	OBJ_DATA *obj_from;
	
	if (obj == NULL) return;
	if ( ( obj_from = obj->in_obj ) == NULL )
	{
		bug( "Obj_from_obj: null obj_from.", 0 );
		return;
	}
	
	if ( obj == obj_from->contains )
	{
		obj_from->contains = obj->next_content;
	}
	else
	{
		OBJ_DATA *prev;
		
		for ( prev = obj_from->contains; prev; prev = prev->next_content )
		{
			if ( prev->next_content == obj )
			{
				prev->next_content = obj->next_content;
				break;
			}
		}
		
		if ( prev == NULL )
		{
			bug( "Obj_from_obj: obj not found.", 0 );
			return;
		}
	}
	
	obj->next_content = NULL;
	obj->in_obj       = NULL;
	
	for ( ; obj_from != NULL; obj_from = obj_from->in_obj )
	{
		if ( obj_from->carried_by != NULL && !IS_SET(obj_from->spectype,SITEM_MORPH))
			obj_from->carried_by->carry_weight -= get_obj_weight( obj );
	}
	
	return;
}



/*
 * Extract an obj from the world.
 */
void extract_obj( OBJ_DATA *obj )
{
	CHAR_DATA *ch;
	OBJ_DATA *obj_content;
	OBJ_DATA *obj_next;
	GSList *tmp = object_list;
	OBJ_DATA *tmpobj;
	GString *buffer;
	
	buffer = g_string_new("");
	if ( obj == NULL ) 
	{
		bug("Object: %d is null!",obj->pIndexData->vnum);
		return;
	}

	if ( obj->in_room != NULL )
		obj_from_room( obj );
	else if ( obj->carried_by != NULL )
		obj_from_char( obj );
	else if ( obj->in_obj != NULL )
		obj_from_obj( obj );
	
	if ( ( ch = obj->chobj ) != NULL
		&&   !IS_NPC(ch)
		&&   ch->pcdata->chobj == obj
		&&   IS_HEAD(ch,LOST_HEAD))
	{
		REMOVE_BIT(ch->loc_hp[0],LOST_HEAD);
		REMOVE_BIT(ch->affected_by,AFF_POLYMORPH);
		ch->morph = g_string_assign(ch->morph,"");
		ch->hit = 1;
		char_from_room(ch);
		char_to_room(ch,get_room_index(ROOM_VNUM_ALTAR));
		ch->pcdata->chobj = NULL;
		obj->chobj = NULL;
		send_to_char("{RYou have been KILLED!!{x\n\r",ch);
		do_look(ch,"auto");
		ch->position = POS_RESTING;
	}
	else if ( ( ch = obj->chobj ) != NULL
		&&   !IS_NPC(ch)
		&&   ch->pcdata->chobj == obj
		&&   (IS_EXTRA(ch,EXTRA_OSWITCH) || ch->pcdata->obj_vnum != 0))
	{
		if (ch->pcdata->obj_vnum != 0)
		{
			send_to_char("You have been destroyed!\n\r",ch);
			ch->pcdata->chobj = NULL;
			obj->chobj = NULL;
		}
		else
		{
			REMOVE_BIT(ch->extra,EXTRA_OSWITCH);
			REMOVE_BIT(ch->affected_by,AFF_POLYMORPH);
			ch->morph = g_string_assign(ch->morph,"");
			char_from_room(ch);
			char_to_room(ch,get_room_index(ROOM_VNUM_ALTAR));
			ch->pcdata->chobj = NULL;
			obj->chobj = NULL;
			send_to_char("You return to your body.\n\r",ch);
		}
	}
	else if ( obj->chobj != NULL )
	{
		if (!IS_NPC(obj->chobj)) 
			obj->chobj->pcdata->chobj = NULL;
		obj->chobj = NULL;
		bug( "Extract_obj: obj %d chobj invalid.", obj->pIndexData->vnum );
	}
	
	for ( obj_content = obj->contains; obj_content; obj_content = obj_next )
	{
		obj_next = obj_content->next_content;
		if (obj->contains != NULL) 
			extract_obj( obj->contains );
	}

	if (object_list == NULL)
	{
		bug( "obj %d caused the object list to dissapear before extract.", obj->pIndexData->vnum );
	}
	
	if (IS_SET(mudsetting->mud_setting,MS_DEBUG)){
		g_string_sprintf(buffer,"Number of Objects in list before: %d\n\r",g_slist_length(object_list));
		bug(buffer->str,0);
	}
	object_list = g_slist_remove(object_list,obj);
	
	if (IS_SET(mudsetting->mud_setting,MS_DEBUG)){
		g_string_sprintf(buffer,"Number of Objects in list after: %d\n\r",g_slist_length(object_list));
		bug(buffer->str,0);
	}
	
	--obj->pIndexData->count;
	
	free_obj(obj);

	if (IS_SET(mudsetting->mud_setting,MS_DEBUG)){
		g_string_sprintf(buffer,"Number of Objects in list after free: %d\n\r",g_slist_length(object_list));
		bug(buffer->str,0);
	}
	
	//Quick check to make sure objects are still sane
	while( tmp != NULL ){
		tmpobj = (OBJ_DATA*)tmp->data;
		
		if (tmpobj == NULL)
		{
			bug("Invalid object detected!\n\r",0);
			tmp = g_slist_remove(tmp,tmpobj);
		}
		
		tmp = g_slist_next(tmp);
	}

	g_slist_free(tmp);
	//After you free the object check the list for NULL. -Spiral 
	g_string_free(buffer,TRUE);

	if (object_list == NULL)
	{
		bug( "obj %d caused the object_list to dissapear after free_obj.", obj->pIndexData->vnum );
	}
	return;
}
/*
 * Find some object with a given index data.
 * Used by area-reset 'P' command.
 */
OBJ_DATA *get_obj_type( OBJ_INDEX_DATA *pObjIndex )
{
	OBJ_DATA *obj;
	GSList *tmp = object_list;
	
	while ( tmp != NULL )
	{
		obj = (OBJ_DATA*)tmp->data;
		if ( obj->pIndexData == pObjIndex )
			return obj;
		tmp = g_slist_next(tmp);
	}
	
	g_slist_free(tmp);
	
	return NULL;
}


/*
 * Find an obj in a list.
 */
OBJ_DATA *get_obj_list( CHAR_DATA *ch, char *argument, OBJ_DATA *list )
{
	char arg[MAX_INPUT_LENGTH];
	OBJ_DATA *obj;
	int number;
	int count;
	
	number = number_argument( argument, arg );
	count  = 0;
	for ( obj = list; obj != NULL; obj = obj->next_content )
	{
		if ( can_see_obj( ch, obj ) && is_name( arg, obj->name->str ) )
		{
			if ( ++count == number )
				return obj;
		}
	}
	
	return NULL;
}



/*
 * Find an object within the object you are in.
 */
OBJ_DATA *get_obj_in_obj( CHAR_DATA *ch, char *argument )
{
	char arg[MAX_INPUT_LENGTH];
	OBJ_DATA *obj;
	OBJ_DATA *obj2;
	int number;
	int count;
	
	number = number_argument( argument, arg );
	count  = 0;
	if (IS_NPC(ch) || ch->pcdata->chobj == NULL || ch->pcdata->chobj->in_obj == NULL)
		return NULL;
	obj = ch->pcdata->chobj;
	if (obj->in_obj->item_type != ITEM_CONTAINER && obj->in_obj->item_type != ITEM_CORPSE_NPC &&
		obj->in_obj->item_type != ITEM_CORPSE_PC ) return NULL;
	
	for ( obj2 = obj->in_obj->contains; obj2 != NULL; obj2 = obj2->next_content )
	{
		if (obj != obj2
			&&  is_name( arg, obj2->name->str ) )
		{
			if ( ++count == number )
				return obj2;
		}
	}
	
	return NULL;
}



/*
 * Find an obj in player's inventory.
 */
OBJ_DATA *get_obj_carry( CHAR_DATA *ch, char *argument )
{
	char arg[MAX_INPUT_LENGTH];
	OBJ_DATA *obj;
	int number;
	int count;
	
	number = number_argument( argument, arg );
	count  = 0;
	for ( obj = ch->carrying; obj != NULL; obj = obj->next_content )
	{
		if ( obj->wear_loc == WEAR_NONE
			&&   can_see_obj( ch, obj )
			&&   is_name( arg, obj->name->str ) )
		{
			if ( ++count == number )
				return obj;
		}
	}
	
	return NULL;
}



/*
 * Find an obj in player's equipment.
 */
OBJ_DATA *get_obj_wear( CHAR_DATA *ch, char *argument )
{
	char arg[MAX_INPUT_LENGTH];
	OBJ_DATA *obj;
	int number;
	int count;
	
	number = number_argument( argument, arg );
	count  = 0;
	for ( obj = ch->carrying; obj != NULL; obj = obj->next_content )
	{
		if ( obj->wear_loc != WEAR_NONE
			&&   can_see_obj( ch, obj )
			&&   is_name( arg, obj->name->str ) )
		{
			if ( ++count == number )
				return obj;
		}
	}
	
	return NULL;
}

void shift_obj_plane(CHAR_DATA *ch)
{
	OBJ_DATA *obj;
	OBJ_DATA *obj2;
	
	for ( obj = ch->carrying; obj != NULL; obj = obj->next_content )
	{
		obj->plane = ch->plane;
		if (obj->contains != NULL)
		{
			for (obj2 = obj->contains; obj2 != NULL; obj2 = obj2->next_content) 
				obj2->plane = ch->plane;
		}
	}
	
	return;
}

/*
 * Find an obj in the room or in inventory.
 */
OBJ_DATA *get_obj_here( CHAR_DATA *ch, char *argument )
{
	OBJ_DATA *obj;
	
	obj = get_obj_list( ch, argument, ch->in_room->contents );
	if ( obj != NULL )
		return obj;
	
	if ( ( obj = get_obj_carry( ch, argument ) ) != NULL )
		return obj;
	
	if ( ( obj = get_obj_wear( ch, argument ) ) != NULL )
		return obj;
	
	if ( ( obj = get_obj_in_obj( ch, argument ) ) != NULL )
		return obj;
	
	return NULL;
}



/*
 * Find an obj in the room or in inventory.
 */
OBJ_DATA *get_obj_room( CHAR_DATA *ch, char *argument )
{
	OBJ_DATA *obj;
	
	obj = get_obj_list( ch, argument, ch->in_room->contents );
	if ( obj != NULL )
		return obj;
	
	return NULL;
}



/*
 * Find an obj in the world.
 */
OBJ_DATA *get_obj_world( CHAR_DATA *ch, char *argument )
{
	char arg[MAX_INPUT_LENGTH];
	OBJ_DATA *obj;
	int number;
	int count;
	GSList *tmp = object_list;
	
	if ( ( obj = get_obj_here( ch, argument ) ) != NULL )
		return obj;
	
	number = number_argument( argument, arg );
	count  = 0;
	while ( tmp != NULL )
	{
		obj = (OBJ_DATA*)tmp->data;
		if ( can_see_obj( ch, obj ) && is_name( arg, obj->name->str ) )
		{
			if ( ++count == number )
				return obj;
		}
		tmp = g_slist_next(tmp);
	}
	g_slist_free(tmp);
	return NULL;
}



/*
 * Create a 'money' obj.
 */
OBJ_DATA *create_money( int amount )
{
	char buf[MAX_STRING_LENGTH];
	OBJ_DATA *obj;
	
	if ( amount <= 0 )
	{
		bug( "Create_money: zero or negative money %d.", amount );
		amount = 1;
	}
	
	if ( amount == 1 )
	{
		obj = create_object( get_obj_index( OBJ_VNUM_MONEY_ONE ), 0 );
	}
	else
	{
		obj = create_object( get_obj_index( OBJ_VNUM_MONEY_SOME ), 0 );
		sprintf( buf, obj->short_descr->str, amount );
		obj->short_descr = g_string_assign(obj->short_descr,buf);
		obj->value[0]		= amount;
	}
	
	return obj;
}

OBJ_DATA *get_obj_world2( CHAR_DATA *ch, char *argument )
{
	OBJ_DATA *obj;
	int vnum = 0;
	GSList *tmp = object_list;
	
	if (argument[0] == '\0') return NULL;
	
	while ( tmp != NULL )
	{
		obj = (OBJ_DATA*)tmp->data;
		
		if ( !str_cmp( argument, obj->short_descr->str ) )
		{
			if ((vnum = obj->pIndexData->vnum) == 30037 || vnum == 30041)
			{
				tmp = g_slist_next(tmp);
				continue;
			}
			return obj;
		}
		tmp = g_slist_next(tmp);
	}
	
	g_slist_free(tmp);
	
	return NULL;
}


/*
 * Return weight of an object, including weight of contents.
 */
int get_obj_weight( OBJ_DATA *obj )
{
	int weight;
	
	weight = obj->weight;
	for ( obj = obj->contains; obj != NULL; obj = obj->next_content )
		weight += get_obj_weight( obj );
	
	return weight;
}
