#include <glib.h>

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <stdio.h>						 
#include <time.h>
#include <merc.h>
#include <tables.h>

const struct rent_type rent_table[]=
{
	{ 29500,"the green planar ring",		800000,100},
	{ 29501,"the blue planar ring",			800000,100},
	{ 29502,"the red planar ring",			800000,100},
	{ 29503,"the chaos axe",		       1500000,150},
	{ 29504,"the torc of continual restoration",   1500000,150},
	{ 29505,"the shortsword of speed",	       1000000,100},
	{ 29506,"the vorpal blade",   		       1000000,100},
	{ 29507,"the klaive",			       1100000,110},
	{ 29508,"the ring of hitting",		       1000000,100},
	{ 29509,"the ring of damage",			750000, 75},
	{ 29510,"the sword of sharpness",	       1000000,100},
	{ 29511,"the banner of peace",			800000,100},
	{ 29512,"the sword of darkness",		800000,100},
	{ 29513,"the tome of the ancients",		800000,100},
	{ 29514,"the armour of resistance",	       1500000,150},
	{ 29515,"the visor of vision",		       1500000,150},
	{ 29516,"the claws of the damned",		750000, 75},
	{ 29517,"the tooth of fenris",		       1500000,150},
	{ 29518,"the bracelet of lloth",	        800000,100},
	{ 29519,"a nameless ring",		       1000000,100},
        { 29520,"the heart of ice",                    1100000,110},
	{ 0,"",0,0}
};

const struct flag_type type_flags[] =
{
	{	"light",		ITEM_LIGHT,		TRUE	},
	{	"scroll",		ITEM_SCROLL,		TRUE	},
	{	"wand",			ITEM_WAND,		TRUE	},
	{	"staff",		ITEM_STAFF,		TRUE	},
	{	"weapon",		ITEM_WEAPON,		TRUE	},
	{	"treasure",		ITEM_TREASURE,		TRUE	},
	{	"armor",		ITEM_ARMOR,		TRUE	},
	{	"potion",		ITEM_POTION,		TRUE	},
	{	"furniture",		ITEM_FURNITURE,		TRUE	},
	{	"trash",		ITEM_TRASH,		TRUE	},
	{	"container",		ITEM_CONTAINER,		TRUE	},
	{	"drinkcontainer",	ITEM_DRINK_CON,		TRUE	},
	{	"key",			ITEM_KEY,		TRUE	},
	{	"food",			ITEM_FOOD,		TRUE	},
	{	"money",		ITEM_MONEY,		TRUE	},
	{	"boat",			ITEM_BOAT,		TRUE	},
	{	"npccorpse",		ITEM_CORPSE_NPC,	TRUE	},
	{	"pc corpse",		ITEM_CORPSE_PC,		FALSE	},
	{	"fountain",		ITEM_FOUNTAIN,		TRUE	},
	{	"pill",			ITEM_PILL,		TRUE	},
	{	"cookie",		ITEM_COOKIE,		TRUE	},
	{	"tool",			ITEM_TOOL,		TRUE	},
	{	"page",		ITEM_PAGE,		TRUE	},
	{	"book",		ITEM_BOOK,		TRUE	},
	{   	"portal",		ITEM_PORTAL,		TRUE	},
	{   	"filet",		ITEM_TFILET,		TRUE	},
	{   	"petfood",		ITEM_PET_FOOD,		TRUE	},
	{   "slotmachine",          ITEM_SLOT_MACHINE,      TRUE    },
    {	NULL,			0,			0	}
};


const struct flag_type extra_flags[] =
{
	{	"glow",			ITEM_GLOW,		TRUE	},
	{	"hum",			ITEM_HUM,		TRUE	},
	{	"thrown",		ITEM_THROWN,		TRUE	},
	{	"keep",			ITEM_KEEP,		TRUE	},
	{	"vanish",		ITEM_VANISH,		TRUE	},
	{	"invis",		ITEM_INVIS,		TRUE	},
	{	"magic",		ITEM_MAGIC,		TRUE	},
	{	"nodrop",		ITEM_NODROP,		TRUE	},
	{	"bless",		ITEM_BLESS,		TRUE	},
	{	"good",			ITEM_ANTI_GOOD,		TRUE	},
	{	"evil",			ITEM_ANTI_EVIL,		TRUE	},
	{	"neutral",		ITEM_ANTI_NEUTRAL,	TRUE	},
	{	"noremove",		ITEM_NOREMOVE,		TRUE	},
	{	"inventory",		ITEM_INVENTORY,		TRUE	},
	{	"loyal",		ITEM_LOYAL,		TRUE	},
	{	"shadowplane",		ITEM_SHADOWPLANE,	TRUE	},
	{	NULL,			0,			0	}
};



const struct flag_type wear_flags[] =
{
	{	"take",			ITEM_TAKE,		TRUE	},
	{	"finger",		ITEM_WEAR_FINGER,	TRUE	},
	{	"neck",			ITEM_WEAR_NECK,		TRUE	},
	{	"body",			ITEM_WEAR_BODY,		TRUE	},
	{	"head",			ITEM_WEAR_HEAD,		TRUE	},
	{	"legs",			ITEM_WEAR_LEGS,		TRUE	},
	{	"feet",			ITEM_WEAR_FEET,		TRUE	},
	{	"hands",		ITEM_WEAR_HANDS,	TRUE	},
	{	"arms",			ITEM_WEAR_ARMS,		TRUE	},
	{	"shield",		ITEM_WEAR_SHIELD,	TRUE	},
	{	"about",		ITEM_WEAR_ABOUT,	TRUE	},
	{	"waist",		ITEM_WEAR_WAIST,	TRUE	},
	{	"wrist",		ITEM_WEAR_WRIST,	TRUE	},
	{	"wield",		ITEM_WIELD,		TRUE	},
	{	"hold",			ITEM_HOLD,		TRUE	},
	{	"face",			ITEM_WEAR_FACE,		TRUE	},
/*    {   "twohands",            ITEM_TWO_HANDS,         TRUE    }, */
	{	NULL,			0,			0	}
};

/*
 * What is seen.
 */
const struct flag_type wear_loc_strings[] =
{
	{	"in the inventory",	WEAR_NONE,	TRUE	},
	{	"as a light",		WEAR_LIGHT,	TRUE	},
	{	"on the left finger",	WEAR_FINGER_L,	TRUE	},
	{	"on the right finger",	WEAR_FINGER_R,	TRUE	},
	{	"around the neck (1)",	WEAR_NECK_1,	TRUE	},
	{	"around the neck (2)",	WEAR_NECK_2,	TRUE	},
	{	"on the body",		WEAR_BODY,	TRUE	},
	{	"over the head",	WEAR_HEAD,	TRUE	},
	{	"on the legs",		WEAR_LEGS,	TRUE	},
	{	"on the feet",		WEAR_FEET,	TRUE	},
	{	"on the hands",		WEAR_HANDS,	TRUE	},
	{	"on the arms",		WEAR_ARMS,	TRUE	},
	{	"as a shield",		WEAR_SHIELD,	TRUE	},
	{	"about the shoulders",	WEAR_ABOUT,	TRUE	},
	{	"around the waist",	WEAR_WAIST,	TRUE	},
	{	"on the left wrist",	WEAR_WRIST_L,	TRUE	},
	{	"on the right wrist",	WEAR_WRIST_R,	TRUE	},
	{	"wielded",		WEAR_WIELD,	TRUE	},
	{	"held in the hands",	WEAR_HOLD,	TRUE	},
	{	NULL,			0	      , 0	}
};


const struct flag_type wear_loc_flags[] =
{
	{	"none",		WEAR_NONE,	TRUE	},
	{	"light",	WEAR_LIGHT,	TRUE	},
	{	"lfinger",	WEAR_FINGER_L,	TRUE	},
	{	"rfinger",	WEAR_FINGER_R,	TRUE	},
	{	"neck1",	WEAR_NECK_1,	TRUE	},
	{	"neck2",	WEAR_NECK_2,	TRUE	},
	{	"body",		WEAR_BODY,	TRUE	},
	{	"head",		WEAR_HEAD,	TRUE	},
	{	"legs",		WEAR_LEGS,	TRUE	},
	{	"feet",		WEAR_FEET,	TRUE	},
	{	"hands",	WEAR_HANDS,	TRUE	},
	{	"arms",		WEAR_ARMS,	TRUE	},
	{	"shield",	WEAR_SHIELD,	TRUE	},
	{	"about",	WEAR_ABOUT,	TRUE	},
	{	"waist",	WEAR_WAIST,	TRUE	},
	{	"lwrist",	WEAR_WRIST_L,	TRUE	},
	{	"rwrist",	WEAR_WRIST_R,	TRUE	},
	{	"wielded",	WEAR_WIELD,	TRUE	},
	{	"face",   	WEAR_FACE, 	TRUE	},
	{	"hold",		WEAR_HOLD,	TRUE	},
	{	NULL,		0,		0	}
};

const struct flag_type container_flags[] =
{
	{	"closeable",		1,		TRUE	},
	{	"pickproof",		2,		TRUE	},
	{	"closed",		4,		TRUE	},
	{	"locked",		8,		TRUE	},
	{	"puton",		16,		TRUE	},
	{	NULL,			0,		0	}
};
