 /***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

/***************************************************************************
*	ROM 2.4 is copyright 1993-1996 Russ Taylor			   				   *
*	ROM has been brought to you by the ROM consortium		   			   *
*	    Russ Taylor (rtaylor@efn.org)				   					   *
*	    Gabrielle Taylor						   						   *
*	    Brian Moore (zump@rom.org)					   					   *
*	By using this code, you have agreed to follow the terms of the	   	   *
*	ROM license, in the file Rom24/doc/rom.license			   			   *
***************************************************************************/
#include <glib.h>

#if defined(macintosh)
#include <types.h>
#else
#include <sys/types.h>
#endif
#include <stdio.h>						 
#include <time.h>
#include <merc.h>
#include <tables.h>
#include <player.h>
#include <power.h>


const struct commands_type hunter_cmds[]=
{
    { TRUE_FAITH,  1,  0, "Heal",             "heal",      250, 24, PTYPE_VICTIM_SELF,           power_heal  }
};

const struct commands_type mage_cmds[]=
{
    { SPHERE_LIFE,  1,  0, "Heal",             "heal",      250, 24, PTYPE_VICTIM_SELF,           power_heal  },
    { SPHERE_LIFE,  2,  0, "Mother's Touch",   "mother",    500, 24, PTYPE_VICTIM_ROOM ,          power_mother},
    { SPHERE_LIFE,  3,  0, "WitherLimb",       "witherlimb",500, 24, PTYPE_OFFENCE_VICTIM_ROOM,   power_witherlimb  },
    { SPHERE_LIFE,  4,  0, "Polymorph",        "polymorph", 500, 24, PTYPE_VICTIM_SELF,           power_fleshcraft  },
    { SPHERE_LIFE,  5,  0, "Life Shield",      "lifeshield",500, 0, PTYPE_AFFECT_SELF,           power_life_shield },
    { SPHERE_LIFE,  6,  0, "Regeneration",     "mregen",   1500, 0, PTYPE_AFFECT_SELF,           power_mregen},
    
    { SPHERE_MIND,  1,  0, "Mana Shield",     "manashield",  500, 0, PTYPE_AFFECT_SELF,           power_mana_shield},
    { SPHERE_MIND,  2,  0, "Pathos",          "pathos",      250, 24, PTYPE_OFFENCE_VICTIM_ROOM,        power_pathos},
    { SPHERE_MIND,  3,  0, "Mind Shield",     "mindshield",  750, 0, PTYPE_AFFECT_SELF,           power_mind_shield},
    { SPHERE_MIND,  4,  0, "Taint Mind",      "taintmind",   750, 24, PTYPE_OFFENCE_VICTIM_ROOM,  power_taintmind},
    { SPHERE_MIND,  5,  0, "Command",         "command",    1000, 6, PTYPE_OFFENCE_VICTIM_ROOM,          power_command},
    { SPHERE_MIND,  6,  0, "Confuse",         "confuse",     500, 12, PTYPE_OFFENCE_VICTIM_ROOM,  power_confuse},
    
    { SPHERE_MATTER,1,  0, "Create Foci",     "createfoci",  500, 24, PTYPE_GENERAL,              power_create_foci},
    { SPHERE_MATTER,2,  0, "Porcupine",       "porcupine",   500, 0, PTYPE_AFFECT_SELF,          power_porcupine},
    { SPHERE_MATTER,3,  0, NULL,              NULL,            0,  0, 0,                          NULL},
    { SPHERE_MATTER,4,  0, "Mist Form",       "mist",        500, 0, PTYPE_AFFECT_SELF,          power_mist},
    { SPHERE_MATTER,5,  0, NULL,              NULL,            0,  0, 0,                          NULL},
    { SPHERE_MATTER,6,  0, "Cloak Equpiment", "cloak",       500, 0, PTYPE_AFFECT_SELF,          power_cloak},
    
    { SPHERE_CORRESPONDENCE,1,  0, "True Sight",      "truesight",   500, 0, PTYPE_AFFECT_SELF,          power_truesight},
    { SPHERE_CORRESPONDENCE,2,  0, "NIght Sight",     "nightsight",  500, 0, PTYPE_AFFECT_SELF,          power_nightsight},
    { SPHERE_CORRESPONDENCE,3,  0, "Shroud",          "shroud",      500, 0, PTYPE_AFFECT_SELF,          power_vanish},
    { SPHERE_CORRESPONDENCE,4,  0, "Location",        "location",    250, 0, PTYPE_VICTIM_GLOBAL,        power_location},
    { SPHERE_CORRESPONDENCE,5,  0, "Ride of the Wild Mind",    "ridewyld", 500, 0, PTYPE_VICTIM_ROOM,          power_ridewyld},
    { SPHERE_CORRESPONDENCE,6,  0, "Dark Sight",      "darksight",   500, 0, PTYPE_AFFECT_SELF,          power_darksight},
    
    { SPHERE_ENTROPY,1,  0, "Shield",          "shield",      500, 0, PTYPE_AFFECT_SELF,          power_shield},
    { SPHERE_ENTROPY,2,  0, NULL,              NULL,          500, 0, PTYPE_AFFECT_SELF,          NULL},
    { SPHERE_ENTROPY,3,  0, "True Fear",       "truefear",    500, 24, PTYPE_OFFENCE_VICTIM_ROOM,  power_true_fear},
    { SPHERE_ENTROPY,4,  0, "Square Circle",   "circle",      500, 0, PTYPE_AFFECT_SELF,          power_squarecircle},
    { SPHERE_ENTROPY,5,  0, NULL,              NULL,          500, 0, PTYPE_AFFECT_SELF,          NULL},
    { SPHERE_ENTROPY,6,  0, "Entropy Shield",  "entropyshield",500, 0, PTYPE_AFFECT_SELF,          power_entropy_shield},
    
    { SPHERE_FORCES, 1,  0, "Charged Bolt",    "bolt",         500, 24, PTYPE_OFFENCE_VICTIM_ROOM,  power_charged_bolt},
    { SPHERE_FORCES, 2,  0, "Aeolus",          "aeolus",       500, 24, PTYPE_GENERAL,              power_aeolus},
    { SPHERE_FORCES, 3,  0, "Rain",            "rain",         500, 24, PTYPE_GENERAL,              power_rain},
    { SPHERE_FORCES, 4,  0, "Force Shield",    "forceshield",  500, 0,  PTYPE_AFFECT_SELF,          power_force_shield},
    { SPHERE_FORCES, 5,  0, "Clap",            "clap",         500, 24, PTYPE_OFFENCE_VICTIM_GLOBAL,power_clap},
    { SPHERE_FORCES, 6,  0, "Walls of Granite","wallgranite",  500, 24, PTYPE_GENERAL,              power_wallgranite},
    
    { SPHERE_PRIME,  1,  0, "Prime Shield",    "primeshield",  500, 0, PTYPE_AFFECT_SELF,              power_prime_shield},
    { SPHERE_PRIME,  2,  0, NULL,              NULL,           500, 24, PTYPE_GENERAL,              NULL},
    { SPHERE_PRIME,  3,  0, NULL,              NULL,           500, 24, PTYPE_GENERAL,              NULL},
    { SPHERE_PRIME,  4,  0, "Flames of Purification","flames",500,  12, PTYPE_OFFENCE_VICTIM_ROOM,  power_flames},
    { SPHERE_PRIME,  5,  0, NULL,              NULL,           500, 24, PTYPE_GENERAL,              NULL},
    { SPHERE_PRIME,  6,  0, "Alter Flow",      "aflow",       1000, 24, PTYPE_OFFENCE_VICTIM_ROOM,  power_alter_flow},
    
    { SPHERE_TIME,   1,  0, "Dispel",          "dispel",       250, 0, PTYPE_AFFECT_SELF,          power_dispel},
    { SPHERE_TIME,   2,  0, "Travel",          "travel",       250, 0, PTYPE_VICTIM_GLOBAL,        power_travel},
    { SPHERE_TIME,   3,  0, "Slow Time",       "slowtime",     250, 12, PTYPE_OFFENCE_VICTIM_ROOM,  power_slow_time},
    { SPHERE_TIME,   4,  0, "Speed Time",      "speedtime",    500, 0, PTYPE_AFFECT_SELF,          power_speed_time},
    { SPHERE_TIME,   5,  0, "Time Sence",      "nocommand",    500, 24, PTYPE_GENERAL,              NULL},
    { SPHERE_TIME,   6,  0, "Rip in Time",     "rip",          500, 12, PTYPE_OFFENCE_VICTIM_ROOM,  power_rip},
    
    { SPHERE_SPIRIT, 1,  0, "Spirit Sight",    "spiritsight",  500, 0, PTYPE_AFFECT_SELF,          power_spirit_sight},
    { SPHERE_SPIRIT, 2,  0, "Spirit Shield",   "spiritshield", 500, 0, PTYPE_AFFECT_SELF,          power_spirit_shield},
    { SPHERE_SPIRIT, 3,  0, "Spirit Plane",    "spiritplane",  500, 0, PTYPE_AFFECT_SELF,          power_spiritplane},
    { SPHERE_SPIRIT, 4,  0, "Summon Spirit",   "summonspirit", 500, 24, PTYPE_GENERAL,           power_summon_spirit},
    { SPHERE_SPIRIT, 5,  0, "Compel",          "compel",       500, 6, PTYPE_OFFENCE_VICTIM_ROOM,  power_compel},
    { SPHERE_SPIRIT, 6,  0, "Soul Stealing",   "stealsoul",    500, 24, PTYPE_GENERAL,              power_steal_soul},
    
    { 0,            0,  0, NULL,                NULL,         0, 24,   0,              NULL}
};

const struct commands_type vamp_cmds1[]=
{
    { DISC_ABOMBWE,	1,	0,	NULL					,	NULL,0, 24,0,NULL	 		},
	{ DISC_ABOMBWE,	2,	0,	NULL					,	NULL,0, 24,0,NULL	 		},
    { DISC_ABOMBWE,	3,	0,	NULL					,	NULL,0, 24,0,NULL	 		},
    { DISC_ABOMBWE,	4,	0,	NULL					,	NULL,0, 24,0,NULL	 		},
    { DISC_ABOMBWE,	5,	0,	NULL					,	NULL,0, 24,0,NULL	 		},
    { DISC_ABOMBWE,	6,	0,	NULL					,	NULL,0, 24,0,NULL	 		},
    { DISC_ABOMBWE,	7,	0,	NULL					,	NULL,0, 24,0,NULL	 		},

    { DISC_ANIMALISM,	1,	0,	"Feral Speech"			,"pigeon",      20, 24,PTYPE_VICTIM_GLOBAL, power_pigeon },
    { DISC_ANIMALISM,	2,	0,	"Noah's Call"			,"beckon",      25, 24,PTYPE_GENERAL,               power_beckon },
    { DISC_ANIMALISM,	3,	0,	"Cowing the Wind"		,"serenity",    25, 24,PTYPE_VICTIM_SELF,   power_serenity },
    { DISC_ANIMALISM,	4,	0,	"Ride the Wild Mind"	,"ridewyld", 25, 24,PTYPE_VICTIM_ROOM,   power_ridewyld },
    { DISC_ANIMALISM,	5,	0,	"Drawing of the Beast"	,"beastwithin", 25, 24,PTYPE_VICTIM_SELF,   power_beastwithin },
    { DISC_ANIMALISM,	6,	0,	"Quickened Unity"		,NULL,0, 24,0,NULL	 		},
    { DISC_ANIMALISM,	7,	0,	NULL					,NULL,0, 24,0,NULL	 		},

    { DISC_AUSPEX,	1,	0,	"Heightened Senses",	"truesight", 0, 0,PTYPE_AFFECT_SELF,           power_truesight    },
    { DISC_AUSPEX,	2,	0,	"Soul Sight"		,	"readaura",  50, 24,PTYPE_OFFENCE_VICTIM_ROOM,	power_readaura 		},
    { DISC_AUSPEX,	3,	0,	"Spirit's Touch"	,	"unveil",    25, 24,PTYPE_VICTIM_SELF,   power_unveil		},
    { DISC_AUSPEX,	4,	0,	"Steal Secrets"		,	"scry",      25, 12,PTYPE_VICTIM_GLOBAL, power_scry	 		},
    { DISC_AUSPEX,	5,	0,	"Anima Walk"		,	"animawalk", 25, 0,PTYPE_VICTIM_GLOBAL,         power_anima     	},
    { DISC_AUSPEX,	6,	0,	"Far Sight"			,	NULL,0, 24,0,NULL	 		},
    { DISC_AUSPEX,	7,	0,	NULL				,	NULL,0, 24,0,NULL	 		},

    { DISC_CELERITY,	1,	0,	"Celerity Power 1"		,	"nocommand",0, 24,0,NULL	 		},
    { DISC_CELERITY,	2,	0,	"Celerity Power 2"		,	"nocommand",0, 24,0,NULL	 		},
    { DISC_CELERITY,	3,	0,	"Celerity Power 3"		,	"nocommand",0, 24,0,NULL	 		},
    { DISC_CELERITY,	4,	0,	"Celerity Power 4"		,	"nocommand",0, 24,0,NULL	 		},
    { DISC_CELERITY,	5,	0,	"Celerity Power 5"		,	"nocommand",0, 24,0,NULL	 		},
    { DISC_CELERITY,	6,	0,	"Celerity Power 6"		,	"nocommand",0, 24,0,NULL	 		},
    { DISC_CELERITY,	7,	0,	"Celerity Power 7"		,	"nocommand",0, 24,0,NULL	 		},
    
    { DISC_CHIMERSTRY,	1,	0,	"Ignis Fatuus"			,	"mirror",           25, 24,PTYPE_VICTIM_ROOM,power_mirror 		},
    { DISC_CHIMERSTRY,	2,	0,	"Fata Morgana"			,	"formillusion",     25, 24,PTYPE_VICTIM_ROOM,power_formillusion },
    { DISC_CHIMERSTRY,	3,	0,	"Apparition"			,	"clone",            20, 24,PTYPE_GENERAL,power_cloneobject 		},
    { DISC_CHIMERSTRY,	4,	0,	"Permanency"			,	"rename",           75, 24,PTYPE_GENERAL,power_rename	},
    { DISC_CHIMERSTRY,	5,	0,	"Horrid Reality"		,	"control",          25, 24,PTYPE_OFFENCE_ROOM,power_control	 		},
    { DISC_CHIMERSTRY,	6,	0,	"Fatuus Mastery"		,	NULL,0, 24,0,NULL	 		},
    { DISC_CHIMERSTRY,	7,	0,	"Far Fatuus"		    ,	NULL,0, 24,0,NULL	 		},


    { DISC_DAIMOINON,	1,	0,	"Sense the Sin"			,	"nocommand",    0, 24,0,NULL	 		},
    { DISC_DAIMOINON,	2,	0,	"Fear the Void Below"	,	"fear",         25, 24,PTYPE_OFFENCE_VICTIM_ROOM, power_fear		},
    { DISC_DAIMOINON,	3,	0,  "Flames of the Netherworld","portal",       10, 24,PTYPE_VICTIM_GLOBAL, power_portal		},
    { DISC_DAIMOINON,	4,	0,	"Psychomachina",	        "soultwist",    30, 24, PTYPE_OFFENCE_VICTIM_ROOM,power_soultwist	 	},
    { DISC_DAIMOINON,	5,	0,	"Curse"					,	"curse",        60, 24, PTYPE_OFFENCE_VICTIM_ROOM,power_curse	 		},
    { DISC_DAIMOINON,	6,	0,	"Diabolic Lure"			,	NULL,0, 24,0,NULL	 		},
    { DISC_DAIMOINON,	7,	0,	"Infernal Servitor" 	,	NULL,0, 24,0,NULL	 		},

    { DISC_DEIMOS,	    1,	0,	"Whispers to the Soul"	,	NULL,0, 24,0,NULL	 		},
    { DISC_DEIMOS,	    2,	0,	"Kiss of the Dark Mother",	NULL,0, 24,0,NULL	 		},
    { DISC_DEIMOS,	    3,	0,	"Ichor"					,	NULL,0, 24,0,NULL	 		},
    { DISC_DEIMOS,	    4,	0,	"Clutching the Shroud"	,	NULL,0, 24,0,NULL	 		},
    { DISC_DEIMOS,	    5,	0,	"Black Breath"			,	NULL,0, 24,0,NULL	 		},
    { DISC_DEIMOS,	    6,	0,	"Lilith's Summons"		,	NULL,0, 24,0,NULL	 		},
      
    { DISC_DEMENTATION,	1,	0,	"Incubus Passion"		,	NULL,0, 24,0,NULL	 		},
    { DISC_DEMENTATION,	2,	0,	"Haunt the Soul"		,	"haunt",25, 24,PTYPE_OFFENCE_VICTIM_ROOM,power_haunt},
    { DISC_DEMENTATION,	3,	0,	"Eyes of Chaos"			,	NULL,0, 24,0,NULL	 		},
    { DISC_DEMENTATION,	4,	0,	"Confusion"				,	"Confusion",50, 24,PTYPE_OFFENCE_VICTIM_ROOM, power_confuse	 		} ,
    { DISC_DEMENTATION,	5,	0,	"Howling Lunacy"		,	NULL,0, 24,0,NULL	 		},
    { DISC_DEMENTATION,	6,	0,	"Insanity"				,	"insane",25, 24,PTYPE_OFFENCE_VICTIM_ROOM,power_insane	 		},
    
    { DISC_DOMINATE,	1,	0,	"Observance of the Spoken Word",	"command", 60, 24,PTYPE_OFFENCE_VICTIM_ROOM,power_command	},
    { DISC_DOMINATE,	2,	0,	"Murmur of the False Will",	        "murmur",  25, 24,PTYPE_OFFENCE_VICTIM_ROOM,power_murmur	},
    { DISC_DOMINATE,	2,	0,	"Shield",               	        "shield",  15, 0,PTYPE_AFFECT_SELF,power_shield	},
    { DISC_DOMINATE,	3,	0,	"Command Familars"		,	        "fcommand", 0, 24,PTYPE_VICTIM_ROOM,power_fcommand	},
    { DISC_DOMINATE,	4,	0,	"Reveler's Memory"		,	        "condition",50, 24,PTYPE_OFFENCE_VICTIM_ROOM,power_condition},
    { DISC_DOMINATE,	5,	0,	"Ghoul"		    		,	        "ghoul",  45, 24,PTYPE_VICTIM_ROOM,power_ghoul	},
    { DISC_DOMINATE,	6,	0,	"Vessel"				,	        "loyalty",45, 24,PTYPE_OFFENCE_VICTIM_ROOM,power_loyalty	 		},
    { DISC_DOMINATE,	7,	0,	NULL					,	NULL,0, 24,0,NULL	 		},

    { DISC_FORTITUDE,	1,	0,	"Fortitude Power 1"		,	"nocommand",0, 24,0,NULL	 		},
    { DISC_FORTITUDE,	2,	0,	"Fortitude Power 2"		,	"nocommand",0, 24,0,NULL	 		},
    { DISC_FORTITUDE,	3,	0,	"Fortitude Power 3"		,	"nocommand",0, 24,0,NULL	 		},
    { DISC_FORTITUDE,	4,	0,	"Fortitude Power 4"		,	"nocommand",0, 24,0,NULL	 		},
    { DISC_FORTITUDE,	5,	0,	"Fortitude Power 5"		,	"nocommand",0, 24,0,NULL	 		},
    { DISC_FORTITUDE,	6,	0,	"Fortitude Power 6"		,	"nocommand",0, 24,0,NULL	 		},
    { DISC_FORTITUDE,	7,	0,	"Fortitude Power 7"		,	"nocommand",0, 24,0,NULL	 		},

    { DISC_GROTESQUOUS,	1,	0,	"Release"				,	NULL,0, 24,0,NULL	 		},
    { DISC_GROTESQUOUS,	2,	0,	"Pariah"				,	NULL,0, 24,0,NULL	 		},
    { DISC_GROTESQUOUS,	3,	0,	"Summoning of the Beast",	NULL,0, 24,0,NULL	 		},
    { DISC_GROTESQUOUS,	4,	0,	"The Push"  			,	NULL,0, 24,0,NULL	 		},
    { DISC_GROTESQUOUS,	5,	0,	"Removal"				,	NULL,0, 24,0,NULL	 		},
    { DISC_GROTESQUOUS,	6,	0,	"Form of the Beast"		,	NULL,0, 24,0,NULL	 		},
    { DISC_GROTESQUOUS,	7,	0,	"The Domain"		    ,	NULL,0, 24,0,NULL	 		},
    
    { DISC_KAI,	1,	0,	"Inner Focus"			,	NULL,0, 24,0,NULL	 		},
    { DISC_KAI,	2,	0,	"Focused Strike"		,	NULL,0, 24,0,NULL	 		},
    { DISC_KAI,	3,	0,	"Honorable Elixir"		,	NULL,0, 24,0,NULL	 		},
    { DISC_KAI,	4,	0,	"Shield from Fire"		,	NULL,0, 24,0,NULL	 		},
    { DISC_KAI,	5,	0,	"Daimyo's Law"  		,	NULL,0, 24,0,NULL	 		},
    { DISC_KAI,	6,	0,	NULL					,	NULL,0, 24,0,NULL	 		},
    { DISC_KAI,	7,	0,	NULL					,	NULL,0, 24,0,NULL	 		},

    { DISC_MELPOMINEE,	1,	0,	"Missing Voice"			,	"scream",30, 24,PTYPE_OFFENCE_VICTIM_ROOM,power_scream		},
    { DISC_MELPOMINEE,	2,	0,	"Tourette's Voice"		,	"reina", 60, 24,PTYPE_AFFECT_SELF,power_reina 		},
    { DISC_MELPOMINEE,	3,	0,	"Toreador's Bane"		,	"courage",40, 24,PTYPE_OFFENCE_VICTIM_ROOM,power_courage 	},
    { DISC_MELPOMINEE,	4,	0,	"Art's Trumatic Essence",	"love",20, 24,PTYPE_OFFENCE_VICTIM_ROOM,power_love	 		},
    { DISC_MELPOMINEE,	5,	0,	"Death of the Drum"		,	"anger",30, 24,PTYPE_OFFENCE_VICTIM_ROOM,power_anger	 	},
    { DISC_MELPOMINEE,	6,	0,	"Blesses Audience"		,	NULL,0, 24,0,NULL	 		},
    { DISC_MELPOMINEE,	7,	0,	"Blesses Audience"		,	NULL,0, 24,0,NULL	 		},

    { DISC_MORTIS,	1,	0,	"Masque of Death"		,	NULL,0, 24,0,NULL	 		},
    { DISC_MORTIS,	2,	0,	"Blight"    			,	NULL,0, 24,0,NULL	 		},
    { DISC_MORTIS,	3,	0,	"Awaken"				,	NULL,0, 24,0,NULL	 		},
    { DISC_MORTIS,	4,	0,	"Death's Whisper"		,	NULL,0, 24,0,NULL	 		},
    { DISC_MORTIS,	5,	0,	"Black Death"			,	NULL,0, 24,0,NULL	 		},
    { DISC_MORTIS,	6,	0,	"Rigor Mortis"			,	NULL,0, 24,0,NULL	 		},
    { DISC_MORTIS,	7,	0,	"Leper's Gait"			,	NULL,0, 24,0,NULL	 		},

    { DISC_NECROMANCY,	1,	0,	"Insight"				,	"thanatopsis",25, 12,PTYPE_GENERAL,power_thanatopsis	},
    { DISC_NECROMANCY,	2,	0,	"Spirit Sight"			,	"spiritsight",25, 12,PTYPE_AFFECT_SELF, power_spirit_sight	 		},
    { DISC_NECROMANCY,	2,	0,	"Spirit Plane"			,	"spiritplane",25, 12,PTYPE_AFFECT_SELF, power_spiritplane	 		},
    { DISC_NECROMANCY,	3,	0,	"Compel"				,	"compel",50, 12,PTYPE_OFFENCE_VICTIM_ROOM,power_compel	 		},
    { DISC_NECROMANCY,	4,	0,	"Haunting"				,	"haunting",25, 12,PTYPE_GENERAL,power_haunting	 		},
    { DISC_NECROMANCY,	5,	0,	"Soul Steal"			,	"stealsoul",75, 12,PTYPE_GENERAL,power_steal_soul	},
    { DISC_NECROMANCY,	6,	0,	"Zombie"				,	"zombie",75, 12,PTYPE_GENERAL,power_zombie	 		},
    { DISC_NECROMANCY,	7,	0,	"Torment"				,	NULL,0, 24,0,NULL	 		},

    { DISC_NIHILISTICS, 1,	0,	"Eyes of the Wraith"	,	NULL,0, 24,0,NULL	 		},
    { DISC_NIHILISTICS,	2,	0,	"Consume the Dead"		,	NULL,0, 24,0,NULL	 		},
    { DISC_NIHILISTICS,	3,	0,	"Aura of Decay"			,	NULL,0, 24,0,NULL	 		},
    { DISC_NIHILISTICS,	4,	0,	"Dark Touch"			,	NULL,0, 24,0,NULL	 		},
    { DISC_NIHILISTICS,	5,	0,	"Night Cry" 			,	NULL,0, 24,0,NULL	 		},
    { DISC_NIHILISTICS,	6,	0,	"Black Blood"			,	NULL,0, 24,0,NULL	 		},
    { DISC_NIHILISTICS,	7,	0,	"Oblivion's Dance"		,	NULL,0, 24,0,NULL	 		},

    { DISC_OBEAH,	1,	0,	"Panacea"				,	"panacea",80, 24,PTYPE_OFFENCE_VICTIM_ROOM,power_panacea	 		},
    { DISC_OBEAH,	2,	0,	"Anaesthetic"			,	NULL,30, 24,PTYPE_OFFENCE_VICTIM_ROOM,power_anesthetic 		},
    { DISC_OBEAH,	3,	0,	"Neutral Guard"			,	"neutral",80, 24,PTYPE_GENERAL,power_neutral	},
    { DISC_OBEAH,	4,	0,	"Treat the Sick Mind"	,	NULL,0, 24,0,NULL	 		},
    { DISC_OBEAH,	5,	0,	"Unburdening of the Bestial Soul",	"unburden",30, 24,PTYPE_OFFENCE_VICTIM_ROOM,power_unburden	 		},
    { DISC_OBEAH,	6,	0,	"Renewed Vigor"     	,	"renew",80, 24,PTYPE_OFFENCE_VICTIM_ROOM,power_renew	 		},
    { DISC_OBEAH,	7,	0,	"Oblivion's Dance"     	,	NULL,0, 24,0,NULL	 		},

    { DISC_OBFUSCATE,	1,	0,	"Cloak of Shadows"		,	"nocommand",0, 24,0,NULL	 		},
    { DISC_OBFUSCATE,	2,	0,	"Unseen Presence"		,	"shield",60, 0,PTYPE_AFFECT_SELF,power_shield 		},
    { DISC_OBFUSCATE,	3,	0,	"Mask of 1000 Faces"	,	"mask",40, 0,PTYPE_VICTIM_SELF,power_mask	        },
    { DISC_OBFUSCATE,	4,	0,	"Vanish from Minds Eye"	,	"mortal",100, 0,PTYPE_AFFECT_SELF,power_mortal     },
    { DISC_OBFUSCATE,	5,	0,	"Cloak the Gathering"	,	"vanish",10, 0,PTYPE_AFFECT_SELF,power_vanish 		},
    { DISC_OBFUSCATE,	6,	0,	"Soul Mask" 			,	"cloak",10, 0,PTYPE_AFFECT_SELF,power_cloak	 		},
    { DISC_OBFUSCATE,	7,	0,	NULL         			,	NULL,0, 24,0,NULL	 		},
    
    { DISC_OBTENEBRATION,	1,	0,	"Shadow Play"			,	"shadowplane",20, 0,PTYPE_AFFECT_SELF,power_shadowplane	 		},
    { DISC_OBTENEBRATION,	2,	0,	"Nocturne"		   		,	"nocturn",60, 24,PTYPE_GENERAL, power_nocturn	},
    { DISC_OBTENEBRATION,	3,	0,	"Arms of Ahriman"		,	"ahriman",30, 24,PTYPE_OFFENCE_VICTIM_ROOM,power_ahriman	 		},
    { DISC_OBTENEBRATION,	4,	0,	"Nightfall" 			,	"nightfall",30, 0,PTYPE_AFFECT_SELF,power_nightfall	 		},
    { DISC_OBTENEBRATION,	5,	0,	"Tenebrous"				,	"tenebrous",60, 0,PTYPE_AFFECT_SELF,power_tenebrous	 		},
    { DISC_OBTENEBRATION,	6,	0,	"Walk the Abyss"		,	"abyss",30, 24,PTYPE_VICTIM_GLOBAL,power_abyss	 		},
    { DISC_OBTENEBRATION,	7,	0,	NULL	            	,	NULL,0, 24,0,NULL 		},
    
    { DISC_OGHAM,	1,	0,	"Consecrate the Grove"	,	NULL,0, 24,0,NULL	 		},
    { DISC_OGHAM,	2,	0,	"Crimson Woad"			,	NULL,0, 24,0,NULL	 		},
    { DISC_OGHAM,	3,	0,	"Inscribe the Curse"	,	NULL,0, 24,0,NULL	 		},
    { DISC_OGHAM,	4,	0,	"Moon and Sun"			,	NULL,0, 24,0,NULL	 		},
    { DISC_OGHAM,	5,	0,	"Dragon Lines"			,	NULL,0, 24,0,NULL	 		},
    { DISC_OGHAM,	6,	0,	"Inscribe the Forgotten Name",	NULL,0, 24,0,NULL	 		},
    { DISC_OGHAM,	7,	0,	NULL					,	NULL,0, 24,0,NULL	 		},
    
    { DISC_POTENCE,	1,	0,	"Potence Power 1"		,	"nocommand",0, 24,0,NULL	 		},
    { DISC_POTENCE,	2,	0,	"Potence Power 2"		,	"nocommand",0, 24,0,NULL	 		},
    { DISC_POTENCE,	3,	0,	"Potence Power 3"		,	"nocommand",0, 24,0,NULL	 		},
    { DISC_POTENCE,	4,	0,	"Potence Power 4"		,	"nocommand",0, 24,0,NULL	 		},
    { DISC_POTENCE,	5,	0,	"Potence Power 5"		,	"flex",0, 12,PTYPE_AFFECT_SELF,power_flex	 		},
    { DISC_POTENCE,	6,	0,	"Potence Power 6"		,	"nocommand",0, 24,0,NULL	 		},
    { DISC_POTENCE,	7,	0,	"Potence Power 7"		,	"nocommand",0, 24,0,NULL	 		},
    
    { DISC_PRESENCE,	1,	0,	"Awe"					,	"awe",40, 24,PTYPE_OFFENCE_VICTIM_ROOM,power_awe	 		},
    { DISC_PRESENCE,	2,	0,	"Dread Gaze"			,	"gaze",25, 24,PTYPE_OFFENCE_VICTIM_ROOM,power_gaze 		},
    { DISC_PRESENCE,	3,	0,	"Entrancement"			,	"entrance",40, 24,PTYPE_OFFENCE_VICTIM_ROOM,power_entrancement	},
    { DISC_PRESENCE,	4,	0,	"Summon"				,	"vsummon",50, 0,PTYPE_VICTIM_GLOBAL,power_vsummon		},
    { DISC_PRESENCE,	5,	0,	"Majesty"				,	"majesty",25, 24,PTYPE_GENERAL,power_majesty		},
    { DISC_PRESENCE,	6,	0,	"Enrage"				,	NULL,0, 24,0,NULL	 		},
    { DISC_PRESENCE,	7,	0,	NULL					,	NULL,0, 24,0,NULL	 		},
    
    { DISC_PROTEAN, 1,	0,	"Witness of Darkness"	,	"nightsight",5, 0,PTYPE_AFFECT_SELF,power_nightsight	 		},
    { DISC_PROTEAN,	2,	0,	"Talon's of the Beast"	,	"claws",5, 0,PTYPE_AFFECT_SELF,power_claws	 		},
    { DISC_PROTEAN,	2,	0,	"Talon's of the Beast"	,	"rage",5, 0,PTYPE_AFFECT_SELF,power_rage	 		},
    { DISC_PROTEAN,	2,	0,	"Talon's of the Beast"	,	"fangs",5, 0,PTYPE_AFFECT_SELF,power_fangs	 		},
    { DISC_PROTEAN,	3,	0,	"Interred in the Earth"	,	"earthmeld",30, 0,PTYPE_AFFECT_SELF,power_earthmeld	},
    { DISC_PROTEAN,	4,	0,	"Form of the Beast"		,	"change",60, 0,PTYPE_AFFECT_SELF,power_change	 		},
    { DISC_PROTEAN,	5,	0,	"Body of the Spirit"	,	"change",60, 0,PTYPE_AFFECT_SELF,power_change   },
    { DISC_PROTEAN,	6,	0,	"Flesh of Marble"		,	"marble",30, 0,PTYPE_AFFECT_SELF,power_marble	},
    { DISC_PROTEAN,	7,	0,	NULL					,	NULL,0, 24,0,NULL	 		},
    
    { DISC_QUIETUS,	1,	0,	"Silence of Death"		,	"silence",25, 24,PTYPE_GENERAL,power_silent	 		},
    { DISC_QUIETUS,	2,	0,	"Weakness"				,	"weakness",25, 24,PTYPE_OFFENCE_VICTIM_ROOM,power_weaken	 		},
    { DISC_QUIETUS,	3,	0,	"Disease"				,	"disease",25, 24,PTYPE_OFFENCE_VICTIM_ROOM,power_disease	 		},
    { DISC_QUIETUS,	4,	0,	"Blood Agony"			,	"agony",25, 24,PTYPE_GENERAL,power_agony	 		},
    { DISC_QUIETUS,	5,	0,	"Taste of Death"		,	"spit",25, 24,PTYPE_OFFENCE_VICTIM_ROOM,power_spit	 		},
    { DISC_QUIETUS,	6,	0,	"Blood Sweat"			,	NULL,25, 24,0,NULL	 		},
    
    { DISC_RIFT,	1,	0,	"The Joining"			,	NULL,0, 24,0,NULL	 		},
    { DISC_RIFT,	2,	0,	"Glimpse of the Pain"	,	NULL,0, 24,0,NULL	 		},
    { DISC_RIFT,	3,	0,	"Crossing the Iron Bridge",	NULL,0, 24,0,NULL	 		},
    { DISC_RIFT,	4,	0,	"Journeying the Iron Bridge",	NULL,0, 24,0,NULL	 		},
    { DISC_RIFT,	5,	0,	"Passing the Inner Realm",	NULL,0, 24,0,NULL	 		},
    { DISC_RIFT,	6,	0,	"Collection Transport"	,	NULL,0, 24,0,NULL	 		},
    { DISC_RIFT,	7,	0,	"Collection Transport"	,	NULL,0, 24,0,NULL	 		},
    
    { DISC_SANGUINUS,	1,	0,	"Brother's Blood"		,	NULL,0, 24,0,NULL	 		},
    { DISC_SANGUINUS,	2,	0,	"Borrow Organs"			,	NULL,0, 24,0,NULL	 		},
    { DISC_SANGUINUS,	3,	0,	"Coordinate Attacks"	,	NULL,0, 24,0,NULL	 		},
    { DISC_SANGUINUS,	4,	0,	"Concentrate Generation",	NULL,0, 24,0,NULL	 		},
    { DISC_SANGUINUS,	5,	0,	"Coagulate Entity"		,	NULL,0, 24,0,NULL	 		},
    { DISC_SANGUINUS,	6,	0,	NULL					,	NULL,0, 24,0,NULL	 		},
    
    { DISC_SERPENTIS,	1,	0,	"Eyes of the Serpent"	,	"nightsight",25, 0, PTYPE_AFFECT_SELF,power_nightsight	 		},
    { DISC_SERPENTIS,	2,	0,	"Tongue of the Asp"		,	NULL,0, 24,0,NULL	 		},
    { DISC_SERPENTIS,	3,	0,	"Bitumenous Flesh"		,	NULL,0, 24,0,NULL	 		},
    { DISC_SERPENTIS,	4,	0,	"Form of the Serpent"	,	"serpent",25, 0,PTYPE_AFFECT_SELF,power_serpent		},
    { DISC_SERPENTIS,	5,	0,	"Heart of Darkness"		,	"darkheart",25, 0,PTYPE_AFFECT_SELF,power_darkheart},
    { DISC_SERPENTIS,	6,	0,	"Breath of the Basilisk",	"breath",40, 24,PTYPE_OFFENCE_VICTIM_ROOM,power_breath},
    
    { DISC_TEMPORIS,	1,	0,	"Time Sence"			,"nocommand",0, 24,0,NULL	 		},
    { DISC_TEMPORIS,	2,	0,	"Ramble On"			,"speedtime",50, 24,PTYPE_AFFECT_SELF,power_speed_time	 		},
    { DISC_TEMPORIS,	3,	0,	"Zombie's Curse"		,"slowtime",50, 12,PTYPE_OFFENCE_VICTIM_ROOM,power_slow_time	 		},
    { DISC_TEMPORIS,	4,	0,	"Cowalker"			,"travel",25, 12,PTYPE_VICTIM_GLOBAL,power_travel	 		},
    { DISC_TEMPORIS,	5,	0,	"Frozen Object"			,"freezeobj",60, 24,PTYPE_GENERAL,power_freeze_object	 		},
    { DISC_TEMPORIS,	6,	0,	"Control Aging"			,	NULL,0, 24,0,NULL	 		},
    { DISC_TEMPORIS,	7,	0,	"Domain of Evernight"	,	NULL,0, 24,0,NULL	 		},
    
    { DISC_THANATOSIS,	1,	0,	"Hag's Wrinkles"		,"hags",25, 12,PTYPE_GENERAL,power_hags_wrinkles	 		},
    { DISC_THANATOSIS,	2,	0,	"Putrefaction"			,	NULL,0, 12,0,NULL	 		},
    { DISC_THANATOSIS,	3,	0,	"Ashes to Ashes"		,"ash",50, 12,PTYPE_AFFECT_SELF,power_ash_to_ash	 		},
    { DISC_THANATOSIS,	4,	0,	"Withering"			,"witherlimb",60, 12,PTYPE_OFFENCE_VICTIM_ROOM,power_witherlimb	},
    { DISC_THANATOSIS,	5,	0,	"Infection" 			,	NULL,0, 24,0,NULL	 		},
    { DISC_THANATOSIS,	6,	0,	"Compress"				,	NULL,0, 24,0,NULL	 		},
    { DISC_THANATOSIS,	7,	0,	"Compress"				,	NULL,0, 24,0,NULL	 		},
                                                                                         
    { DISC_THAUMATURGY,	1,	0,	"Taste"					,	"taste",30, 12,PTYPE_OFFENCE_VICTIM_ROOM,power_tasteblood	    },
    { DISC_THAUMATURGY,	2,	0,	"Thaumaturgic Countermagic","counter",30, 0,PTYPE_VICTIM_SELF,power_counterspell	 		},
    { DISC_THAUMATURGY,	3,	0,	"Bloodrage"				,	"bloodrage",20, 24,PTYPE_OFFENCE_VICTIM_ROOM,power_bloodrage 	},
    { DISC_THAUMATURGY,	4,	0,	"Potency"				,	"potency",20, 24,PTYPE_AFFECT_SELF,power_potency	 		},
    { DISC_THAUMATURGY,	5,	0,	"Theft"					,	"theft",0, 24,PTYPE_OFFENCE_VICTIM_ROOM,power_theftvitae	 		},
    { DISC_THAUMATURGY,	6,	0,	"Cauldron"				,	"cauldron",20, 24,PTYPE_OFFENCE_VICTIM_ROOM,power_cauldronblood 		},
    { DISC_THAUMATURGY,	7,	0,	NULL    				,	NULL,0, 24,0,NULL	 		},
    
    { DISC_VICISSITUDE,	1,	0,	"Changeling"			,	"changeling",25, 0,PTYPE_VICTIM_SELF,power_changeling	},
    { DISC_VICISSITUDE,	2,	0,	"Fleshcraft"			,	"fleshcraft",50, 0,PTYPE_VICTIM_SELF,power_fleshcraft	},
    { DISC_VICISSITUDE,	3,	0,	"Bonecraft"				,	"bonecraft",60, 24,PTYPE_GENERAL,power_bonecraft		},
    { DISC_VICISSITUDE,	4,	0,	"Horrid Form"			,	"zuloform",60, 0,PTYPE_AFFECT_SELF,power_zuloform	 		},
    { DISC_VICISSITUDE,	5,	0,	"Inner Essance"			,	"essence",60, 0,PTYPE_AFFECT_SELF,power_essence	 		},
    { DISC_VICISSITUDE,	6,	0,	"Blood of Acid"			,	"nocommand",0, 24,0,NULL	 		},
    { DISC_VICISSITUDE,	7,	0,	NULL        			,	NULL,0, 24,0,NULL	 		},
    
    { DISC_VISCERATIKA,	1,	0,	"Whispers of the Chamber",	"discern",20, 0,  PTYPE_AFFECT_SELF, power_discern      },
    { DISC_VISCERATIKA,	2,	0,	"Skin of the Chameleon"	,	"chameleon",20, 0,PTYPE_AFFECT_SELF,power_chameleon     },
    { DISC_VISCERATIKA,	3,	0,	"Voices of the Castle"	,	"vcastle",20, 0,PTYPE_AFFECT_SELF,power_voices_of_castle},
    { DISC_VISCERATIKA,	4,	0,	"Bond with Terra"		,	"bterra",30, 0,PTYPE_AFFECT_SELF,power_bond_terra 		},
    { DISC_VISCERATIKA,	5,	0,	"Stone Strength"		,	"marble",30, 0,PTYPE_AFFECT_SELF,power_marble 		    },
    { DISC_VISCERATIKA,	6,	0,	"Rockheart, Dark Statue",	"nocommand",40, 0,PTYPE_AFFECT_SELF,NULL	 	},
    { DISC_VISCERATIKA,	7,	0,	"Crawling Chamber"		,	"wallgranite",60, 24, PTYPE_GENERAL,power_wallgranite	},
    
    
    { 0 ,	0,	0,	NULL					,	NULL,0, 24,0,NULL	 		}

};

const struct commands_type wwf_tot_cmds[]=
{
	{ WPOWER_MANTIS,	1,	0,	"Fast Attacks"			,	"nocommand",0, 24,0,NULL	 		},
	{ WPOWER_MANTIS,	2,	0,	"Very Fast Attacks"		,	"nocommand",0, 24,0,NULL	 		},
    { WPOWER_MANTIS,	3,	0,	"Extremely Fast Attacks",	"nocommand",0, 24,0,NULL	 		},
    { WPOWER_MANTIS,	4,	0,  "Incredibly Fast Attacks",	"nocommand",0, 24,0,NULL	 		},
    
    { WPOWER_BEAR,	1,	0,	"Flex"					,	"flex",0, 0,PTYPE_AFFECT_SELF,power_flex	 		},
	{ WPOWER_BEAR,	2,	0,	"Rage"					,	"rage",0, 0,PTYPE_AFFECT_SELF,power_rage	 		},
    { WPOWER_BEAR,	3,	0,	"Steel Claws"			,	"nocommand",0, 24,0,NULL	 		},
    { WPOWER_BEAR,	4,	0,	"Hibernation"   		,	"nocommand",0, 24,0,NULL	 		},
    
    { WPOWER_LYNX,	1,	0,	"Light Footed"			,	"nocommand",0, 24,0,NULL	 		},
	{ WPOWER_LYNX,	2,	0,	"Stalker"				,	"hunt",0, 12,PTYPE_VICTIM_GLOBAL,power_hunt	 		},
    { WPOWER_LYNX,	3,	0,	"Combat Speed"			,	"nocommand",0, 24,0,NULL	 		},
    { WPOWER_LYNX,	4,	0,	"Lightning Claws"		,	"nocommand",0, 24,0,NULL	 		},
    
    { WPOWER_BOAR ,	1,	0,	"Shatter"   			,	"nocommand",0, 24,0,NULL	 		},
	{ WPOWER_BOAR ,	2,	0,	"Charge"				,	"nocommand",0, 24,0,NULL	 		},
    { WPOWER_BOAR ,	3,	0,	"Toughness"				,	"nocommand",0, 24,0,NULL	 		},
    { WPOWER_BOAR ,	4,	0,	"Immovability"			,	"nocommand",0, 24,0,NULL	 		},
    
    { WPOWER_OWL ,	1,	0,	"Vanish"				,	"vanish",0, 0,PTYPE_AFFECT_SELF,power_vanish	 		},
	{ WPOWER_OWL ,	2,	0,	"Shield"				,	"shield",0, 0,PTYPE_AFFECT_SELF,power_shield	 		},
    { WPOWER_OWL ,	3,	0,	"ShadowPlane"			,	"shadowplane",0, 0,PTYPE_AFFECT_SELF,power_shadowplane	},
    { WPOWER_OWL ,	4,	0,	"Magic Control"			,	"nocommand",0, 24,0,NULL	 		},
    
    { WPOWER_SPIDER ,	1,	0,	"Poisonous bite" 		,	"nocommand",0, 24,0,NULL	 		},
	{ WPOWER_SPIDER ,	2,	0,	"Web"					,	"web",0, 24,PTYPE_OFFENCE_VICTIM_ROOM,power_web	 		},
    { WPOWER_SPIDER ,	3,	0,	"Immunity to Poison"  	,	"nocommand",0, 24,0,NULL	 		},
    
    { WPOWER_WOLF ,	1,	0,	"Claws"					,	"claws",0, 0,PTYPE_AFFECT_SELF,power_claws		},
	{ WPOWER_WOLF ,	2,	0,	"Fangs"					,	"fangs",0, 0,PTYPE_AFFECT_SELF,power_fangs		},
    { WPOWER_WOLF ,	3,	0,	"Calm"					,	"calm",0, 24,PTYPE_AFFECT_SELF,power_calm		},
    { WPOWER_WOLF ,	4,	0,	"Spirit of Fenris"		,	"nocommand",0, 24,0,NULL	 		},
    
    { WPOWER_HAWK ,	1,	0,	"NightSight"			,	"nightsight",0, 0,PTYPE_AFFECT_SELF,power_nightsight		},
	{ WPOWER_HAWK ,	2,	0,	"ShadowSight"			,	"shadowsight",0, 0,PTYPE_AFFECT_SELF,power_shadowsight		},
    { WPOWER_HAWK ,	3,	0,	"True Sight"			,	"truesight",0, 0,PTYPE_AFFECT_SELF,power_truesight	 		},
    
    { WPOWER_SHARK ,	1,	0,	"Vicious Jaws"			,	"nocommand",0, 24,0,NULL	 		},
	{ WPOWER_SHARK ,	2,	0,	"Gnawing Bite"			,	"nocommand",0, 24,0,NULL	 		},
    { WPOWER_SHARK ,	3,	0,	"Clenching Jaws"		,	"nocommand",0, 24,0,NULL	 		},
    { WPOWER_SHARK ,	4,	0,	"Bite of Fenris"  		,	"nocommand",0, 24,0,NULL	 		},
    { WPOWER_SHARK ,	5,	0,	"Slashing Wound"  		,	"nocommand",0, 24,0,NULL	 		},
     
    { WPOWER_SPHINX ,	1,	0,	"Moon Gate"				,	"moongate",0, 0,PTYPE_VICTIM_GLOBAL,power_moongate	 		},
	{ WPOWER_SPHINX ,	2,	0,	"Griffin's Tail"		,	"nocommand",0, 24,0,NULL	 		},
    { WPOWER_SPHINX ,	3,	0,	"Moon Beam"				,	"moonbeam",0, 24,PTYPE_OFFENCE_VICTIM_ROOM,power_moonbeam	 		},
    { WPOWER_SPHINX ,	4,	0,	"Precision of Battle"	,	"nocommand",0, 24,0,NULL	 		},
    { WPOWER_SPHINX ,	5,	0,	"Luna's Armor"  		,	"nocommand",0, 0,0,NULL	 		},

    { WPOWER_RAT ,	1,	0,	"Burrow"					,	"burrow",0, 24,PTYPE_VICTIM_GLOBAL,power_burrow	 		},
    { WPOWER_RAT ,	2,	0,	"Quills"					,	"nocommand",0, 24,0,NULL	 		},
    { WPOWER_RAT ,	3,	0,	"Speed of the Rat"			,	"nocommand",0, 24,0,NULL	 		},
    { WPOWER_RAT ,	4,	0,	"Pounce"					,	"pounce",0, 24,PTYPE_OFFENCE_VICTIM_ROOM,power_pounce	 		},
    { WPOWER_RAT ,	5,	0,	"Prowl"	    				,	"nocommand",0, 24,0,NULL	 		},
	
    { WPOWER_CHAMELEON ,	1,	0,	"Mirror Image"			,	"mirror",0, 24,PTYPE_GENERAL, power_mirror	    },
	{ WPOWER_CHAMELEON ,	2,	0,	"Stab in the Dark"		,	"nocomamnd",0, 24,0,NULL	 		},
    { WPOWER_CHAMELEON ,	3,	0,	"Gift of the Spriggan"	,	"spriggan",0, 0,PTYPE_AFFECT_SELF,power_spriggan	},
    { WPOWER_CHAMELEON ,	4,	0,	"Instinct"  			,	"nocommand",0, 24,0,NULL	 		},
    { WPOWER_CHAMELEON ,	5,	0,	"Speed of Thought"		,	"nocommand",0, 24,0,NULL	 		},

    { 0 ,	0,	0,	NULL					,	NULL,0, 24,0,NULL	 		}

};

/*
const struct commands_type wraith_cmds[]=
{
	{ ARCONOI_ARGOS, 1,	0, "shadowsight", "shadowsight",50,24,0,power_shadowsight },	
	{ ARCONOI_ARGOS, 2,	0, "shadowplane", "shadowplane",50,24,0,power_shadowplane },
	{ ARCONOI_ARGOS, 3,	0, "locate"	, "locate",100,24,0,power_locate },
	{ ARCONOI_ARGOS, 4,	0, "travel"	, "travel",100,12,0,power_travel },
	{ ARCONOI_ARGOS, 5,	0, "pathos"	, "scry",150,24,0,power_scry },	
	{ ARCONOI_ARGOS, 6,	0, "vsummon"	, "vsummon",200,24,0,power_vsummon },	

	{ ARCONOI_CASTIGATE, 1,	0, "Blue Wraith"	, NULL,0,24,0,NULL },//Mana Mod	
	{ ARCONOI_CASTIGATE, 2,	0, "Purple Wraith"	, NULL,0,24,0,NULL },//AC Mod	
	{ ARCONOI_CASTIGATE, 3,	0, "Silver Wraith"	, NULL,0,24,0,NULL },//Silver Bodie	
	{ ARCONOI_CASTIGATE, 4,	0, "Red Wraith"	, NULL,0,24,0,NULL },//Hp Mod	
	{ ARCONOI_CASTIGATE, 5,	0, "Green Wraith"	, NULL,0,24,0,NULL },//Hitroll mod	
	{ ARCONOI_CASTIGATE, 6,	0, "Cyan Wraith"	, NULL,0,24,0,NULL },//Damroll Mod	

	{ ARCONOI_EMBODY, 1, 0, "", NULL, 0, 24, 0, NULL}, 
	{ ARCONOI_EMBODY, 2, 0, "", NULL, 0, 24, 0, NULL},
	{ ARCONOI_EMBODY, 3, 0, "", NULL, 0, 24, 0, NULL},
	{ ARCONOI_EMBODY, 4, 0, "", NULL, 0, 24, 0, NULL},
	{ ARCONOI_EMBODY, 5, 0, "", NULL, 0, 24, 0, NULL},
	{ ARCONOI_EMBODY, 6, 0, "", NULL, 0, 24, 0, NULL},	
};
*/
const struct commands_type vamp_cmds2[]=
{
	{ 0 ,	0,	0,	NULL					,	NULL,0, 24,0,NULL	 		}
};

const struct commands_type wwf_gift_cmds[]=
{
	{ BREED_LUPUS,	1,	1, "Heighten"			,	"truesight",  20, 24, PTYPE_AFFECT_SELF,  power_truesight	},
	{ BREED_LUPUS,	2,	1, "Noah's Call"		,	"beckon",     30, 24, PTYPE_GENERAL,      power_beckon	    },
	{ BREED_LUPUS,	3,	1, "Master the Animals"	,	"animal",     20, 24, PTYPE_AFFECT_SELF,  power_animal	    },
	{ BREED_LUPUS,	4,	1, "Ride the Wild Mind"	,	"ridewyld",30, 24, PTYPE_VICTIM_ROOM,  power_ridewyld },
	{ BREED_LUPUS,	5,	1, "Flesh Bond"			,	"fleshbond", 100, 24, PTYPE_GENERAL,  power_fleshbond    	},
	
	{ BREED_HOMID,	1,	1, "Persuasion"	   		,	"persuasion", 50, 24, PTYPE_OFFENCE_VICTIM_ROOM,power_persuasion },
	{ BREED_HOMID,	2,	1, "Staredown"	   		,	"staredown",  50, 24, PTYPE_OFFENCE_VICTIM_ROOM,power_staredown },
	{ BREED_HOMID,	3,	1, "Reshape Object"		,	"reshape",   100, 24, PTYPE_GENERAL,       power_reshape_object        },
	{ BREED_HOMID,	4,	1, "Disquiet"	   		,	"disquiet",   50, 24, PTYPE_OFFENCE_VICTIM_ROOM,power_disquiet },
	{ BREED_HOMID,	5,	1, "Assimilation"  		,	"mask",       60, 24, PTYPE_VICTIM_ROOM,   power_mask     },
	
	{ BREED_METIS,	1,	1, "Sense Wyrm"	 		,	"hunt",       15, 0, PTYPE_VICTIM_GLOBAL,power_hunt     },
	{ BREED_METIS,	2,	1, "Burrow"	  			,	"spburrow",   40, 24, PTYPE_AFFECT_SELF,   power_spburrow },
	{ BREED_METIS,	3,	1, "Madness"	  		,	"madness",    50, 24, PTYPE_OFFENCE_VICTIM_ROOM,power_confuse  },
	{ BREED_METIS,	4,	1, "Wither Limbs"	  	,	"witherlimb",100, 24, PTYPE_OFFENCE_VICTIM_ROOM,power_witherlimb },
	{ BREED_METIS,	5,	1, "Gift of the Porcupine",	"porcupine",  30, 24, PTYPE_AFFECT_SELF,   power_porcupine    },
	
	{ AUSPICE_GALLIARD,	1,	2, "Call of the Wyld",	"wyld",        0, 24, PTYPE_GENERAL,       power_wyld        },
	{ AUSPICE_GALLIARD,	2,	2, "Dreamspeak"		 ,	"dreamspeak", 50, 24, PTYPE_VICTIM_ROOM,   power_haunt  },
	{ AUSPICE_GALLIARD,	3,	2, "Beast Within"	 ,	"beastwithin",50, 24, PTYPE_VICTIM_ROOM,power_beastwithin },
	{ AUSPICE_GALLIARD,	4,	2, "Moon Bridge"	 ,	"moonbridge", 30, 0, PTYPE_VICTIM_GLOBAL,power_moon_bridge },
	{ AUSPICE_GALLIARD,	5,	2, "Head Games"		 ,	"command",   100, 24, PTYPE_OFFENCE_RORP,  power_command     },
	
	{ AUSPICE_RAGABASH,	1,	2, "Blur of the Milky Eye", "blur",   50, 24, PTYPE_AFFECT_SELF,   power_blur   },
	{ AUSPICE_RAGABASH,	2,	2, "Scent of the Running Water"	,"nocommand",0, 24,  PTYPE_GENERAL,   NULL    },
	{ AUSPICE_RAGABASH, 3,	2, "Luna's Blessing"  	,	"nocommand",0, 24, PTYPE_GENERAL,   NULL    },
	{ AUSPICE_RAGABASH,	4,	2, "Tail of the Monkey",	"tail",   50, 24, PTYPE_OFFENCE_VICTIM_ROOM,power_tail },
	{ AUSPICE_RAGABASH,	5,	2, "Violation"			,	"condition",50, 24, PTYPE_OFFENCE_VICTIM_ROOM,power_condition },
	
	{ AUSPICE_AHROUN,	1,	2, "Razor Claws" 		,	"nocommand", 0, 24, PTYPE_GENERAL,       NULL        },
	{ AUSPICE_AHROUN,	2,	2, "Falling Touch"		,	"ftouch",   50, 24, PTYPE_OFFENCE_VICTIM_ROOM,power_ftouch },
	{ AUSPICE_AHROUN,	3,	2, "True Fear"			,	"truefear", 50, 24, PTYPE_OFFENCE_VICTIM_ROOM,power_true_fear },
	{ AUSPICE_AHROUN,	4,	2, "Clenched Jaw"		,	"nocommand", 0, 24, PTYPE_GENERAL,       NULL         },
	{ AUSPICE_AHROUN,	5,	2, "Flaming Hand"		,	"hand",     20, 24, PTYPE_AFFECT_SELF,   power_hand 	 },
	
	//Theurge 3 needs to be changed..... they already have shadowsight with 2..
    { AUSPICE_THEURGE,	1,	2, "Mother's Touch"		,	"mother",       75, 24, PTYPE_VICTIM_ROOM,power_mother         },
	{ AUSPICE_THEURGE,	2,	2, "Scent from Beyond"	,	"shadowsight",  20, 24, PTYPE_AFFECT_SELF,power_shadowsight	 },
	{ AUSPICE_THEURGE,	3,	2, "Pulse of the Invisable","spiritsight",  20, 24, PTYPE_AFFECT_SELF,power_spirit_sight 	 },
	{ AUSPICE_THEURGE,	4,	2, "Grasp the Beyond"	,	"nocommand",     0, 24, PTYPE_GENERAL,    NULL        },
	{ AUSPICE_THEURGE,	5,	2, "Feral Lobotomy"		,	"insane",       50, 24, PTYPE_OFFENCE_VICTIM_ROOM,power_insane },
	
	{ AUSPICE_PHILODOX,	1,	2, "Call to Duty" 		,	"callduty",     60, 24, PTYPE_GENERAL,       power_call_duty      },
	{ AUSPICE_PHILODOX,	2,	2, "Scry"				,	"scry",         50, 24, PTYPE_VICTIM_GLOBAL,power_scry           },
	{ AUSPICE_PHILODOX,	3,	2, "Roll Over"			,	"rollover",     50, 24, PTYPE_OFFENCE_VICTIM_ROOM,power_rollover       },
	{ AUSPICE_PHILODOX,	4,	2, "Scent from Beyond"	,	"shadowsight",  20, 24, PTYPE_AFFECT_SELF,   power_shadowsight    },
	{ AUSPICE_PHILODOX,	5,	2, "Wall of Granite"	,	"wallgranite",  40, 24, PTYPE_GENERAL,       power_wallgranite    },
	
	{ TRIBE_BLACK_FURIES,	1,	3, "Curse of Aeolus"	,	"aeolus",   30, 24, PTYPE_GENERAL,       power_aeolus        },
	{ TRIBE_BLACK_FURIES,	2,	3, "Wasp Talons"	    ,	"wtalon",   50, 24, PTYPE_OFFENCE_VICTIM_ROOM,power_wtalon },
	{ TRIBE_BLACK_FURIES,	3,	3, "Gorgon's Gaze"	    ,	"gorgon",   50, 24, PTYPE_OFFENCE_VICTIM_ROOM,power_gorgon },
	{ TRIBE_BLACK_FURIES,	4,	3, "Bacchantes's Rage"	,	"brage",   100, 24, PTYPE_AFFECT_SELF,   power_brage    },
	{ TRIBE_BLACK_FURIES,	5,	3, "Body Wrack"			,	"bodywrack",50, 24, PTYPE_OFFENCE_VICTIM_ROOM,power_bodywrack },
	
	{ TRIBE_FIANNA,	1,	3, "Resist Toxin" 	    ,	"nocommand",    0, 24, PTYPE_GENERAL,               NULL           },
	{ TRIBE_FIANNA,	2,	3, "Howl of the Banshee",	"howlban",     50, 12, PTYPE_OFFENCE_VICTIM_ROOM,        power_howlban    },
	{ TRIBE_FIANNA,	3,	3, "Illusionary Form" 	,	"formillusion",80, 24, PTYPE_VICTIM_ROOM,           power_formillusion},
	{ TRIBE_FIANNA,	4,	3, "Clone Object" 		,	"cloneobject",100, 24, PTYPE_GENERAL,               power_cloneobject           },
	{ TRIBE_FIANNA,	5,	3, "Gift of the Spriggan",	"spgift",      20, 24, PTYPE_AFFECT_SELF,           power_spgift       },
	
	{ TRIBE_SHADOW_LORDS,	1,	3, "Aura of Confidence"	,	"sheild",    20, 24, PTYPE_AFFECT_SELF,       power_shield   },
	{ TRIBE_SHADOW_LORDS,	2,	3, "Clap of Thunder"	,	"clap",       100, 24, PTYPE_OFFENCE_VICTIM_GLOBAL,power_clap },
	{ TRIBE_SHADOW_LORDS,	3,	3, "Icy Chill of Despair",	"icy",         120, 48, PTYPE_AFFECT_SELF,power_icy},
	{ TRIBE_SHADOW_LORDS,	4,	3, "Paralyzing Stare"	,	"paralyzing",  50, 24, PTYPE_OFFENCE_VICTIM_ROOM,power_paralyzing},
	{ TRIBE_SHADOW_LORDS,	5,	3, "Obedience"			,	"obedience",  100, 24, PTYPE_OFFENCE_RORP,  power_obedience},
	
	{ TRIBE_GET_OF_FENRIS,	1,	3, "Venom Blood"		,	"nocommand",    0, 24, PTYPE_GENERAL,       NULL   },
	{ TRIBE_GET_OF_FENRIS,	2,	3, "Scream"		    	,	"scream",      60, 24, PTYPE_OFFENCE_VICTIM_ROOM,power_scream},
	{ TRIBE_GET_OF_FENRIS,	3,	3, "Halt Cowards Flight",	"web",         40, 24, PTYPE_OFFENCE_VICTIM_ROOM,power_web},
	{ TRIBE_GET_OF_FENRIS,	4,	3, "Fenris Bite"		,	"fbite",       50, 24, PTYPE_OFFENCE_VICTIM_ROOM,power_fbite},
	{ TRIBE_GET_OF_FENRIS,	5,	3, "Berserkers Song"	,	"bsong",        0, 24, PTYPE_AFFECT_SELF,   power_bsong},
	
    { TRIBE_SILVER_FANGS,	1,	3, "Lambent Flame" 		,	"lflame",      20, 24, PTYPE_AFFECT_SELF,   power_lflame   },
	{ TRIBE_SILVER_FANGS,	2,	3, "Wraith of Spiral" 	,	"spwrath",     50, 24, PTYPE_OFFENCE_VICTIM_ROOM,power_spwrath},
	{ TRIBE_SILVER_FANGS,	3,	3, "MindBlock" 		    ,	"mindblock",   20, 24, PTYPE_AFFECT_SELF,   power_mindblock   },
	{ TRIBE_SILVER_FANGS,	4,	3, "Mass Summons" 		,	"masssummons", 80, 24, PTYPE_GENERAL,       power_mass_summons},
	{ TRIBE_SILVER_FANGS,	5,	3, "Luna's Avenger" 	,	"lunaavenger", 30, 24, PTYPE_AFFECT_SELF,   power_lunaavenger   },
	
    { TRIBE_SILENT_STRIDERS,	1,	3, "Attunement" 		    ,	"attune",      40, 12, PTYPE_AFFECT_SELF,   power_attune   },
	{ TRIBE_SILENT_STRIDERS,	2,	3, "Messangers Fortitude" 	,	"nocommand",     50, 24, PTYPE_OFFENCE_VICTIM_ROOM,NULL},
	{ TRIBE_SILENT_STRIDERS,	3,	3, "Eyes of Ma'at" 		    ,	"scry",    25, 12,   PTYPE_VICTIM_GLOBAL,   power_scry   },
	{ TRIBE_SILENT_STRIDERS,	4,	3, "Gate of the Moon" 		,   "moongate", 30, 24, PTYPE_VICTIM_GLOBAL,       power_moongate},
	{ TRIBE_SILENT_STRIDERS,	5,	3, "Speed Beyond Thought"   ,   "nocommand", 30, 24, PTYPE_AFFECT_SELF,  NULL   },
	
	{ TRIBE_STARGAZERS,	1,	3, "WaterWalking" 		,	"nocommand",        0, 24, PTYPE_GENERAL,       NULL       },
	{ TRIBE_STARGAZERS,	2,	3, "-" 		,	"-",0, 24, PTYPE_GENERAL,NULL 	},
	{ TRIBE_STARGAZERS,	3,	3, "-" 		,	"-",0, 24, PTYPE_GENERAL,NULL 	},
	{ TRIBE_STARGAZERS,	4,	3, "-" 		,	"-",0, 24, PTYPE_GENERAL,NULL 	},
	{ TRIBE_STARGAZERS,	5,	3, "-" 		,	"-",0, 24, PTYPE_GENERAL,NULL 	},
	
	{ TRIBE_UKTENA,	1,	3, "Shroud of Night" 	,	"shroudnight",  20, 24, PTYPE_AFFECT_SELF,           power_shroudnight 	},
	{ TRIBE_UKTENA,	2,	3, "WaterWalking" 		,	"nocommand",     0, 24, PTYPE_GENERAL,               NULL    	},
	{ TRIBE_UKTENA,	3,	3, "ReadAura"   		,	"readaura",     50, 24, PTYPE_VICTIM_GLOBAL,         power_readaura 		},
	{ TRIBE_UKTENA,	4,	3, "Invisibility" 		,	"invisibility", 60, 24, PTYPE_AFFECT_SELF,           power_invisibilty   },
	{ TRIBE_UKTENA,	5,	3, "Fetish Doll" 		,	"fetishdoll",   70, 24, PTYPE_VICTIM_GLOBAL, power_fetishdoll       },
	
	{ TRIBE_WENDIGO,	1,	3, "Camouflage" 		,	"camouflage",20, 24, PTYPE_AFFECT_SELF,          power_camouflage	    },
	{ TRIBE_WENDIGO,	2,	3, "Wall of Water" 		,	"wall",      50, 24, PTYPE_GENERAL,              power_wall		    },
	{ TRIBE_WENDIGO,	3,	3, "Lightning" 			,	"lightning", 50, 24, PTYPE_OFFENCE_VICTIM_ROOM,  power_lightning 	},
	{ TRIBE_WENDIGO,	4,	3, "Eclipse" 			,	"eclipse",   60, 24, PTYPE_GENERAL,              power_eclipse 		    },
	{ TRIBE_WENDIGO,	5,	3, "Heart of Ice"  		,	"hice",     200, 24, PTYPE_OFFENCE_VICTIM_ROOM,  power_hice         },
	
	{ TRIBE_CHILD_OF_SPIRAL,   1,	3, "Mother's Touch"		,	"mother",   75, 24, PTYPE_VICTIM_ROOM,   power_mother	},
	{ TRIBE_CHILD_OF_SPIRAL,	2,	3, "Spiral's Tears"		,	"rain",     50, 24, PTYPE_GENERAL,       power_rain   },
	{ TRIBE_CHILD_OF_SPIRAL,	3,	3, "Dazzle"				,	"dazzle",   50, 24, PTYPE_OFFENCE_VICTIM_ROOM,power_dazzle },
	{ TRIBE_CHILD_OF_SPIRAL,	4,	3, "Cowing the Beast"	,	"serenity", 40, 24, PTYPE_OFFENCE_VICTIM_ROOM,power_serenity },
	{ TRIBE_CHILD_OF_SPIRAL,	5,	3, "Halo of the Sun"	,	"halosun",  70, 24, PTYPE_OFFENCE_VICTIM_ROOM,power_halosun },
	{ 0					   ,	0,	0, NULL					,	NULL,       0,  0	 		}
};

       
const struct dumb_shit_type dumb_shit[]=
{
	{ "advance"   },
	{ "delete"    },
	{ "gift"      },
	{ "practice"  },
	{ "sacrifice" },
	{ "consent"   },
	{ "nosum"     },
	{ "train"     },
	{ "kill"      },
	{ "password"  },
	{ "token"     },
	{ "call"      },
	{ "quit"      },
	{ "drop"      },
	{ ""          }  /* ending comamnd */
};
/* for position */

const struct msp_type msp_midi_table[]=
{
	{ "sweetdreams.mid", MSP_SERVER, MSP_MIDDIR },
	{ "BURN.MID", MSP_SERVER, MSP_MIDDIR },
	{ "ERASER.MID", MSP_SERVER, MSP_MIDDIR },
	{ "CLOSER.MID", MSP_SERVER, MSP_MIDDIR },
	{ "DEADSOULS.MID", MSP_SERVER, MSP_MIDDIR },
	{ "DOWNINIT.MID", MSP_SERVER, MSP_MIDDIR },
	{ "MARCHPIG.MID", MSP_SERVER, MSP_MIDDIR },
	{ "NOTHING.MID", MSP_SERVER, MSP_MIDDIR },
	{ "PIGGY.MID", MSP_SERVER, MSP_MIDDIR },
	{ "BECOMING.MID", MSP_SERVER, MSP_MIDDIR }
};

/* Msp scream tables to be silly.. */
const struct msp_type msp_scream_table[]=
{
	{ "scream_1.wav", MSP_SERVER, MSP_WAVDIR },
	{ "scream_2.wav", MSP_SERVER, MSP_WAVDIR },
	{ "scream_3.wav", MSP_SERVER, MSP_WAVDIR },
	{ "scream_4.wav", MSP_SERVER, MSP_WAVDIR },
	{ "death_cry1.wav", MSP_SERVER, MSP_WAVDIR },
	{ "death_cry2.wav", MSP_SERVER, MSP_WAVDIR },
	{ "death_cry3.wav", MSP_SERVER, MSP_WAVDIR },
	{ "death_cry4.wav", MSP_SERVER, MSP_WAVDIR }
};

/* This is to just create a standard of MSP sounds.. callem by number.. 
 * instead of random.(so that you can program by number and have the wav in one place
 - Spiral
 */
const struct msp_type msp_general_table[]=
{
	{ "clock.wav", MSP_SERVER, MSP_WAVDIR },      /*0*/
	{ "decay.wav", MSP_SERVER, MSP_WAVDIR },
	{ "portal.wav", MSP_SERVER, MSP_WAVDIR },
	{ "runaway.wav", MSP_SERVER, MSP_WAVDIR },
	{ "thunder1.wav", MSP_SERVER, MSP_WAVDIR },
	{ "mount.wav", MSP_SERVER, MSP_WAVDIR },
	{ "pscream.wav", MSP_SERVER, MSP_WAVDIR },
	{ "creak.wav", MSP_SERVER, MSP_WAVDIR },
	{ "punch.wav", MSP_SERVER, MSP_WAVDIR },      /*8*/
	{ "undertaker.wav", MSP_SERVER, MSP_WAVDIR },
	{ "backstab.wav", MSP_SERVER, MSP_WAVDIR },
	{ "huh.wav", MSP_SERVER, MSP_WAVDIR },
	{ "crickets.wav", MSP_SERVER, MSP_WAVDIR },
	{ "rooster.wav", MSP_SERVER, MSP_WAVDIR },    /*13*/
	{ "guilotine.wav", MSP_SERVER, MSP_WAVDIR },
	{ "whokill.wav", MSP_SERVER, MSP_WAVDIR }     /*15*/
};


const struct position_type position_table[] =
{
	{	"dead",			"dead"	},
	{	"mortally wounded",	"mort"	},
	{	"incapacitated",	"incap"	},
	{	"stunned",		"stun"	},
	{	"sleeping",		"sleep"	},
	{	"meditating",	"meditate"	},
	{   "sitting",		"sit"   },
	{	"resting",		"rest"	},
	{	"fighting",		"fight"	},
	{	"standing",		"stand"	},
	{	NULL,			NULL	}
};

/* for sex */
const struct sex_type sex_table[] =
{
	{	"none"		},
	{	"male"		},
	{	"female"	},
	{	"either"	},
	{	NULL		}
};

const struct flag_type mprog_flags[] =
{
	{	"act",			TRIG_ACT,		TRUE	},
	{	"bribe",		TRIG_BRIBE,		TRUE 	},
	{	"death",		TRIG_DEATH,		TRUE    },
	{	"entry",		TRIG_ENTRY,		TRUE	},
	{	"fight",		TRIG_FIGHT,		TRUE	},
	{	"give",			TRIG_GIVE,		TRUE	},
	{	"greet",		TRIG_GREET,		TRUE    },
	{	"grall",		TRIG_GRALL,		TRUE	},
	{	"kill",			TRIG_KILL,		TRUE	},
	{	"hpcnt",		TRIG_HPCNT,		TRUE    },
	{	"random",		TRIG_RANDOM,		TRUE	},
	{	"speech",		TRIG_SPEECH,		TRUE	},
	{	"exit",			TRIG_EXIT,		TRUE    },
	{	"exall",		TRIG_EXALL,		TRUE    },
	{	"delay",		TRIG_DELAY,		TRUE    },
	{	"surr",			TRIG_SURR,		TRUE    },
	{	"riddle",		TRIG_RIDD,		TRUE    },
	{	NULL,			0,			0    }
};

const struct flag_type mprog_obj_flags[] =
{
	{	"act",			TRIG_ACT,		TRUE	},
	{	"death",		TRIG_DEATH,		TRUE    },
	{	"entry",		TRIG_ENTRY,		TRUE	},
	{	"fight",		TRIG_FIGHT,		TRUE	},
	{	"random",		TRIG_RANDOM,	TRUE	},
	{	"exit",			TRIG_EXIT,		TRUE    },
	{	"delay",		TRIG_DELAY,		TRUE    },
	{	"push",			TRIG_PUSH,		TRUE    },
	{	"pull",			TRIG_PULL,		TRUE    },
	{	"activate",		TRIG_ACTIVATE, 	TRUE    },
	{	"twist",		TRIG_TWIST, 	TRUE    },
	{	"look",			TRIG_LOOK,	 	TRUE    },
	{	NULL,			0,			0    }
};

const struct flag_type mprog_room_flags[] =
{
	{	"act",			TRIG_ACT,		TRUE	},
	{	"death",		TRIG_DEATH,		TRUE    },
	{	"entry",		TRIG_ENTRY,		TRUE	},
	{	"fight",		TRIG_FIGHT,		TRUE	},
	{	"random",		TRIG_RANDOM,	TRUE	},
	{	"exit",			TRIG_EXIT,		TRUE    },
	{	"delay",		TRIG_DELAY,		TRUE    },
	{	NULL,			0,			0    }
};

/* for sizes */
const struct size_type size_table[] =
{ 
	{	"tiny"		},
	{	"small" 	},
	{	"medium"	},
	{	"large"		},
	{	"huge", 	},
	{	"giant" 	},
	{	NULL		}
};

/* various flag tables */
const struct flag_type mob_fight_flags[]=
{
	{	"viper",	A,	TRUE 	},
	{	"crane",	B,	TRUE 	},
	{	"mongoose",	C,	TRUE 	},
	{	"bull",		D,	TRUE 	},
	{	"crab",		E,	TRUE 	},
	{	"mantis",	F,	TRUE 	},
	{	"tiger",	G,	TRUE 	},
	{	"dragon",	H,	TRUE 	},
	{	"monkey",	I,	TRUE 	},
	{	"swallow",	J,	TRUE 	},
	{	"random",	K,	TRUE 	},
	{	"master",	L,	TRUE 	},
	{	"none",	    0,  TRUE	},
	{	NULL,	    0,  FALSE	}
};

const struct flag_type act_flags[] =
{
	{	"npc",			A,	FALSE	},
	{	"sentinel",		B,	TRUE	},
	{	"scavenger",		C,	TRUE	},
	{	"aggressive",		D,	TRUE	},
	{	"stay_area",		E,	TRUE	},
	{	"wimpy",		F,	TRUE	},
	{	"pet",			G,	TRUE	},
	{	"train",		J,	TRUE	},
	{	"practice",		K,	TRUE	},
	{	"noexp",		L,	TRUE	},
	{	"holylight",		M,	TRUE	},
	{	"undead",		O,	TRUE	},
	{	"plrkill",		P,	TRUE	},
	{	"hunter",		Q,	TRUE	},
	{	"animal",		R,	TRUE	},
	{	"human",		S,	TRUE	},
	{	"guard",		T,	TRUE	},
	{	"elite",		U,	TRUE	},
	{	"nopurge",		V,	TRUE	},
	{	"outdoors",		W,	TRUE	},
	{	"indoors",		Y,	TRUE	},
	{	"healer",		aa,	TRUE	},
	{	"gain",			bb,	TRUE	},
	{	"update_always",	cc,	TRUE	},
	{	"changer",		dd,	TRUE	},
	{	NULL,			0,	FALSE	}
};

const struct flag_type plr_flags[] =
{
	{	"npc",			A,	FALSE	},
	{	"autoassist",		C,	FALSE	},
	{	"autoexit",		D,	FALSE	},
	{	"autoloot",		E,	FALSE	},
	{	"autosac",		F,	FALSE	},
	{	"autogold",		G,	FALSE	},
	{	"autosplit",		H,	FALSE	},
	{	"holylight",		N,	FALSE	},
	{	"can_loot",		P,	FALSE	},
	{	"nosummon",		Q,	FALSE	},
	{	"nofollow",		R,	FALSE	},
	{	"permit",		U,	TRUE	},
	{	"log",			W,	FALSE	},
	{	"deny",			X,	FALSE	},
	{	"freeze",		Y,	FALSE	},
	{	"thief",		Z,	FALSE	},
	{	"killer",		aa,	FALSE	},
	{	NULL,			0,	0	}
};

const struct flag_type affect_flags[] =
{
	{	"blind",		A,	TRUE	},
	{	"invisible",		B,	TRUE	},
	{	"detect_evil",		C,	TRUE	},
	{	"detect_invis",		D,	TRUE	},
	{	"detect_magic",		E,	TRUE	},
	{	"detect_hidden",	F,	TRUE	},
	{	"shadowplane",		G,	TRUE	},
	{	"sanctuary",		H,	TRUE	},
	{	"faerie_fire",		I,	TRUE	},
	{	"infrared",		J,	TRUE	},
	{	"curse",		K,	TRUE	},
	{	"flaming",              L,	TRUE	},
	{	"poison",		M,	TRUE	},
	{	"protect",		N,	TRUE	},
	{	"ethereal",    		O,	TRUE	},
	{	"sneak",		P,	TRUE	},
	{	"hide",			Q,	TRUE	},
	{	"sleep",		R,	TRUE	},
	{	"charm",		S,	TRUE	},
	{	"flying",		T,	TRUE	},
	{	"pass_door",		U,	TRUE	},
	{	"polymorph",            V,	TRUE	},
	{	"shadowsight",		W,	TRUE	},
	{	"webbed",		X,	TRUE	},
	{	"contraception",	Y,	TRUE	},
	{	"darkness",		Z,	TRUE	},
	{	"disease",		aa,	TRUE	},
	{	"aggdam",       	bb,	TRUE	},
	{	NULL,			0,	0	}
};

const struct flag_type off_flags[] =
{
	{	"area_attack",		A,	TRUE	},
	{	"backstab",		B,	TRUE	},
	{	"bash",			C,	TRUE	},
	{	"berserk",		D,	TRUE	},
	{	"disarm",		E,	TRUE	},
	{	"dodge",		F,	TRUE	},
	{	"fade",			G,	TRUE	},
	{	"fast",			H,	TRUE	},
	{	"kick",			I,	TRUE	},
	{	"dirt_kick",		J,	TRUE	},
	{	"parry",		K,	TRUE	},
	{	"rescue",		L,	TRUE	},
	{	"tail",			M,	TRUE	},
	{	"trip",			N,	TRUE	},
	{	"crush",		O,	TRUE	},
	{	"assist_all",		P,	TRUE	},
	{	"assist_align",		Q,	TRUE	},
	{	"assist_race",		R,	TRUE	},
	{	"assist_players",	S,	TRUE	},
	{	"assist_guard",		T,	TRUE	},
	{	"assist_vnum",		U,	TRUE	},
	{	NULL,			0,	0	}
};

const struct flag_type imm_flags[] =
{
	{	"summon",		A,	TRUE	},
	{	"charm",		B,	TRUE	},
	{	"magic",		C,	TRUE	},
	{	"weapon",		D,	TRUE	},
	{	"bash",			E,	TRUE	},
	{	"pierce",		F,	TRUE	},
	{	"slash",		G,	TRUE	},
	{	"fire",			H,	TRUE	},
	{	"cold",			I,	TRUE	},
	{	"lightning",		J,	TRUE	},
	{	"acid",			K,	TRUE	},
	{	"poison",		L,	TRUE	},
	{	"negative",		M,	TRUE	},
	{	"holy",			N,	TRUE	},
	{	"energy",		O,	TRUE	},
	{	"mental",		P,	TRUE	},
	{	"disease",		Q,	TRUE	},
	{	"drowning",		R,	TRUE	},
	{	"light",		S,	TRUE	},
	{	"sound",		T,	TRUE	},
	{	"wood",			X,	TRUE	},
	{	"silver",		Y,	TRUE	},
	{	"iron",			Z,	TRUE	},
	{	NULL,			0,	0	}
};

const struct flag_type area_flags[] =
{
	{	"none",			AREA_NONE,		FALSE	},
	{	"changed",		AREA_CHANGED,		TRUE	},
	{	"added",		AREA_ADDED,		TRUE	},
	{	"loading",		AREA_LOADING,		FALSE	},
	{	NULL,			0,			0	}
};



const struct flag_type sex_flags[] =
{
	{	"male",			SEX_MALE,		TRUE	},
	{	"female",		SEX_FEMALE,		TRUE	},
	{	"neutral",		SEX_NEUTRAL,		TRUE	},
	{   "random",               3,                      TRUE    },   /* ROM */
	{	"none",			SEX_NEUTRAL,		TRUE	},
	{	NULL,			0,			0	}
};



const struct flag_type exit_flags[] =
{
	{   "door",			EX_ISDOOR,		TRUE    },
	{	"closed",		EX_CLOSED,		TRUE	},
	{	"locked",		EX_LOCKED,		TRUE	},
	{	"pickproof",		EX_PICKPROOF,		TRUE	},
	{	"passproof",		EX_PASSPROOF,		TRUE	},
	{	NULL,			0,			0	}
};



const struct flag_type door_resets[] =
{
	{	"open and unlocked",	0,		TRUE	},
	{	"closed and unlocked",	1,		TRUE	},
	{	"closed and locked",	2,		TRUE	},
	{	NULL,			0,		0	}
};



const struct flag_type room_flags[] =
{
	{	"dark",			ROOM_DARK,		TRUE	},
	{	"no_mob",		ROOM_NO_MOB,		TRUE	},
	{	"indoors",		ROOM_INDOORS,		TRUE	},
	{	"private",		ROOM_PRIVATE,		TRUE    },
	{	"safe",			ROOM_SAFE,		TRUE	},
	{	"solitary",		ROOM_SOLITARY,		TRUE	},
	{	"pet_shop",		ROOM_PET_SHOP,		TRUE	},
	{	"no_recall",		ROOM_NO_RECALL,		TRUE	},
	{	NULL,			0,			0	}
};



const struct flag_type sector_flags[] =
{
	{	"inside",	SECT_INSIDE,		TRUE	},
	{	"city",		SECT_CITY,		TRUE	},
	{	"field",	SECT_FIELD,		TRUE	},
	{	"forest",	SECT_FOREST,		TRUE	},
	{	"hills",	SECT_HILLS,		TRUE	},
	{	"mountain",	SECT_MOUNTAIN,		TRUE	},
	{	"swim",		SECT_WATER_SWIM,	TRUE	},
	{	"noswim",	SECT_WATER_NOSWIM,	TRUE	},
	{   "unused",	SECT_UNUSED,		TRUE	},
	{	"air",		SECT_AIR,		TRUE	},
	{	"desert",	SECT_DESERT,		TRUE	},
	{	"tundra",	SECT_TUNDRA,		TRUE	},
	{	NULL,		0,			0	}
};




/*
 * Used when adding an affect to tell where it goes.
 * See addaffect and delaffect in act_olc.c
 */
const struct flag_type apply_flags[] =
{
	{	"none",			APPLY_NONE,		TRUE	},
	{	"strength",		APPLY_STR,		TRUE	},
	{	"dexterity",		APPLY_DEX,		TRUE	},
	{	"intelligence",		APPLY_INT,		TRUE	},
	{	"wisdom",		APPLY_WIS,		TRUE	},
	{	"constitution",		APPLY_CON,		TRUE	},
	{	"sex",			APPLY_SEX,		TRUE	},
	{	"class",		APPLY_CLASS,		TRUE	},
	{	"level",		APPLY_LEVEL,		TRUE	},
	{	"age",			APPLY_AGE,		TRUE	},
	{	"height",		APPLY_HEIGHT,		TRUE	},
	{	"weight",		APPLY_WEIGHT,		TRUE	},
	{	"mana",			APPLY_MANA,		TRUE	},
	{	"hp",			APPLY_HIT,		TRUE	},
	{	"move",			APPLY_MOVE,		TRUE	},
	{	"gold",			APPLY_GOLD,		TRUE	},
	{	"experience",		APPLY_EXP,		TRUE	},
	{	"ac",			APPLY_AC,		TRUE	},
	{	"hitroll",		APPLY_HITROLL,		TRUE	},
	{	"damroll",		APPLY_DAMROLL,		TRUE	},
	{	NULL,			0,			0	}
};

const struct flag_type mob_con_flags[] =
{
	{ "reslight", MOBCON_RESLIGHT, TRUE },
	{ "teacher" , MOBCON_TEACHER, TRUE },
	{ "domestic", MOBCON_DOMESTIC, TRUE },
	{ "dealer",   MOBCON_DEALER, TRUE },
	{ NULL	       , 0                 , 0 }
};


/*****************************************************************************
                      ROM - specific tables:
 ****************************************************************************/



const struct flag_type position_flags[] =
{
	{   "dead",           POS_DEAD,            FALSE   },
	{   "mortal",         POS_MORTAL,          FALSE   },
	{   "incap",          POS_INCAP,           FALSE   },
	{   "stunned",        POS_STUNNED,         FALSE   },
	{   "sleeping",       POS_SLEEPING,        TRUE    },
	{   "resting",        POS_RESTING,         TRUE    },
	{   "sitting",        POS_SITTING,         TRUE    },
	{   "fighting",       POS_FIGHTING,        FALSE   },
	{   "standing",       POS_STANDING,        TRUE    },
	{   NULL,              0,                    0       }
};

/*
 * Used when adding an affect to tell where it goes.
 * See addaffect and delaffect in act_olc.c
 */
const struct flag_type apply_types[] =
{
	{	"none",			APPLY_NONE,		TRUE	},
	{	"strength",		APPLY_STR,		TRUE	},
	{	"dexterity",		APPLY_DEX,		TRUE	},
	{	"intelligence",		APPLY_INT,		TRUE	},
	{	"wisdom",		APPLY_WIS,		TRUE	},
	{	"constitution",		APPLY_CON,		TRUE	},
	{	"sex",			APPLY_SEX,		TRUE	},
	{	"class",		APPLY_CLASS,		TRUE	},
	{	"level",		APPLY_LEVEL,		TRUE	},
	{	"age",			APPLY_AGE,		TRUE	},
	{	"height",		APPLY_HEIGHT,		TRUE	},
	{	"weight",		APPLY_WEIGHT,		TRUE	},
	{	"mana",			APPLY_MANA,		TRUE	},
	{	"hp",			APPLY_HIT,		TRUE	},
	{	"move",			APPLY_MOVE,		TRUE	},
	{	"gold",			APPLY_GOLD,		TRUE	},
	{	"experience",		APPLY_EXP,		TRUE	},
	{	"ac",			APPLY_AC,		TRUE	},
	{	"hitroll",		APPLY_HITROLL,		TRUE	},
	{	"damroll",		APPLY_DAMROLL,		TRUE	},
	{	"savingpara",		APPLY_SAVING_PARA,	TRUE	},
	{	"savingrod",		APPLY_SAVING_ROD,	TRUE	},
	{	"savingpetri",		APPLY_SAVING_PETRI,	TRUE	},
	{	"savingbreath",		APPLY_SAVING_BREATH,	TRUE	},
	{	"savingspell",		APPLY_SAVING_SPELL,	TRUE	},
	{	NULL,			0,			0	}
};

const	struct	bit_type	bitvector_type	[]	=
{
	{	affect_flags,	"affect"	},
	{	apply_flags,	"apply"		}
};

const struct flag_type furniture_flags[]=
{
    {	"none",		  0,			TRUE	},
    {   "stand_at",	  STAND_AT,		TRUE	},
    {	"stand_on",	  STAND_ON,		TRUE	},
    {	"stand_in",	  STAND_IN,		TRUE	},
    {	"sit_at",	  SIT_AT,		TRUE	},
    {	"sit_on",	  SIT_ON,		TRUE	},
    {	"sit_in",	  SIT_IN,		TRUE	},
    {	"rest_at",	  REST_AT,		TRUE	},
    {	"rest_on",	  REST_ON,		TRUE	},
    {	"rest_in",	  REST_IN,		TRUE	},
    {	"sleep_at",	  SLEEP_AT,		TRUE	},
    {	"sleep_on",	  SLEEP_ON,		TRUE	},
    {	"sleep_in",	  SLEEP_IN,		TRUE	},
    {	"put_at",	  PUT_AT,		TRUE	},
    {	"put_on",	  PUT_ON,		TRUE	},
    {	"put_in",	  PUT_IN,		TRUE	},
    {	"put_inside",	  PUT_INSIDE,		TRUE	},
    {	NULL,		  0,			0	}
};


const struct flag_type attack_table[] =
{
	
	{	"hit",			DAMAGE_HIT,			TRUE 		},
	{	"slice",		DAMAGE_SLICE,		TRUE		},
	{	"stab",			DAMAGE_STAB,		TRUE		},
	{	"slash",		DAMAGE_SLASH,		TRUE		},
	{	"whip",			DAMAGE_WHIP,		TRUE		},
	{	"claw",			DAMAGE_CLAW,		TRUE		},
	{	"blast",		DAMAGE_BLAST,		TRUE		},
	{	"pound",		DAMAGE_POUND,		TRUE		},
	{	"crush", 		DAMAGE_CRUSH,		TRUE		},
	{	"grep",			DAMAGE_GREP,		TRUE		},
	{	"bite",			DAMAGE_BITE,		TRUE		},
	{	"pierce",		DAMAGE_PIERCE,		TRUE		},
	{	"suck",			DAMAGE_SUCK,		TRUE		},
	{	"light beam",    DAMAGE_LIGHT_BEAM,	TRUE		},
    {	NULL,		  	0,					0			}
};

const struct trait_type trait_table[]=
{
	{	"Lucky",	MERIT_LUCKY,		3,	TRUE, MERIT },
	{	"Wise",	    MERIT_WISE,			7,	TRUE, MERIT },
	{   "Tough", 	MERIT_TOUGH,		6,	TRUE, MERIT },
	{	"Willfull", MERIT_WILLFULL,		7,	TRUE, MERIT },
	{   "Studious", MERIT_STUDIOUS,		6,	TRUE, MERIT },
	{   "Agile",	MERIT_AGILE,		5,	TRUE, MERIT },
	{   "Perceptive",MERIT_PERCEPTIVE,	5,	TRUE, MERIT },
	
	{	"Unlucky",	FLAW_UNLUCKY,		3,	TRUE, FLAW },
	{	"Foolish",	FLAW_FOOLISH,		7,	TRUE, FLAW  },
	{   "Fragile",  FLAW_FRAGILE,		6,  TRUE, FLAW  },
	{   "WeakWilled",FLAW_WEAKWILLED,	7,  TRUE, FLAW  },
	{   "Dumb",		FLAW_DUMB,			6,	TRUE, FLAW  },
	{	"Clumsy",	FLAW_CLUMSY,		5,	TRUE, FLAW  },
	{	"Unperceptive",FLAW_UNPERCEPTIVE, 5,	TRUE, FLAW	},
	
	{	"Slash",	IMM_MERIT_SLASH,	5,	TRUE, IM_MERIT },
	{   "Stab", 	IMM_MERIT_STAB, 	5,	TRUE, IM_MERIT },
	{   "Smash", 	IMM_MERIT_SMASH,	5,	TRUE, IM_MERIT },
	{   "Animal",	IMM_MERIT_ANIMAL,	5,	TRUE, IM_MERIT },
	{   "Misc",		IMM_MERIT_MISC,		5,	TRUE, IM_MERIT },
	{   "Charm",	IMM_MERIT_CHARM,	5,  TRUE, IM_MERIT },
	{   "Heat",		IMM_MERIT_HEAT,		5,	TRUE, IM_MERIT },
	{	"Cold",		IMM_MERIT_COLD,		5,	TRUE, IM_MERIT },
	{   "Lightning",IMM_MERIT_LIGHTNING,5,  TRUE, IM_MERIT },
	{   "Acid",		IMM_MERIT_ACID,		5,	TRUE, IM_MERIT },
	{   "Voodoo",	IMM_MERIT_VOODOO,	5,	TRUE, IM_MERIT },
	{   "Hurl",		IMM_MERIT_HURL,		5,  TRUE, IM_MERIT },
	{	"Backstab",	IMM_MERIT_BACKSTAB,	5,	TRUE, IM_MERIT },
	{	"Kick",		IMM_MERIT_KICK,		5, 	TRUE, IM_MERIT },
	{   "Disarm",	IMM_MERIT_DISARM,	5,	TRUE, IM_MERIT },
	{   "Steal",	IMM_MERIT_STEAL,	5,	TRUE, IM_MERIT },
	{   "Sleep",	IMM_MERIT_SLEEP,	5,	TRUE, IM_MERIT },
	{	"Drain",	IMM_MERIT_DRAIN,	5, 	TRUE, IM_MERIT },
	
	{	"Slash",	IMM_FLAW_SLASH,		5,	TRUE, IM_FLAW 	},
	{   "Stab", 	IMM_FLAW_STAB, 		5,	TRUE, IM_FLAW 	},
	{   "Smash", 	IMM_FLAW_SMASH,		5,	TRUE, IM_FLAW 	},
	{   "Animal",	IMM_FLAW_ANIMAL,	5,	TRUE, IM_FLAW 	},
	{   "Misc",		IMM_FLAW_MISC,		5,	TRUE, IM_FLAW 	},
	{   "Charm",	IMM_FLAW_CHARM,		5,  TRUE, IM_FLAW 	},
	{   "Heat",		IMM_FLAW_HEAT,		5,	TRUE, IM_FLAW 	},
	{	"Cold",		IMM_FLAW_COLD,		5,	TRUE, IM_FLAW 	},
	{   "Lightning",IMM_FLAW_LIGHTNING, 5,  TRUE, IM_FLAW 	},
	{   "Acid",		IMM_FLAW_ACID,		5,	TRUE, IM_FLAW 	},
	{   "Voodoo",	IMM_FLAW_VOODOO,	5,	TRUE, IM_FLAW 	},
	{   "Hurl",		IMM_FLAW_HURL,		5,  TRUE, IM_FLAW 	},
	{	"Backstab",	IMM_FLAW_BACKSTAB,	5,	TRUE, IM_FLAW 	},
	{	"Kick",		IMM_FLAW_KICK,		5, 	TRUE, IM_FLAW 	},
	{   "Disarm",	IMM_FLAW_DISARM,	5,	TRUE, IM_FLAW 	},
	{   "Steal",	IMM_FLAW_STEAL,		5,	TRUE, IM_FLAW 	},
	{   "Sleep",	IMM_FLAW_SLEEP,		5,	TRUE, IM_FLAW 	},
	{	"Drain",	IMM_FLAW_DRAIN,		5, 	TRUE, IM_FLAW 	},
	
	{	NULL,	0,	0,	0, 0 } 
}; 	


const struct damage_type damage_table[] =
{
	{ "hit",        "hits",     DAMAGE_HIT	        },     
	{ "slice",      "slices",   DAMAGE_SLICE	    },
	{ "stab",       "stabs",    DAMAGE_STAB	        },
	{ "slash",      "slashes",  DAMAGE_SLASH	    },
	{ "whip",       "whips",    DAMAGE_WHIP	        },
	{ "claw",       "claws",    DAMAGE_CLAW	        },
	{ "blast",      "blasts",   DAMAGE_BLAST	    },
	{ "pound",      "pounds",   DAMAGE_POUND	    },
	{ "crush",      "crushes",  DAMAGE_CRUSH	    },
	{ "grep",       "greps",    DAMAGE_GREP	        },
	{ "bite",       "bites",    DAMAGE_BITE	        },
	{ "pierce",     "pierces",  DAMAGE_PIERCE	    },
	{ "suck",       "sucks",    DAMAGE_SUCK	        },
	{ "light beam", "beams",    DAMAGE_LIGHT_BEAM	},
	{ "bash",       "bashes",   DAMAGE_BASH 	    },
	{ "elbow",      "elbows",   DAMAGE_ELBOW 	    },
	{ "knee",       "knees",    DAMAGE_KNEE 	    },
	{ "headbutt",   "headbutts",DAMAGE_HEADBUTT     },
	{ "grapple",    "grapples", DAMAGE_GRAPPLE      },
	{ "rip",        "rips",     DAMAGE_RIP   	    },
	{ "kick",       "kicks",    DAMAGE_KICK 	    },
	{ "charge",     "charges",  DAMAGE_CHARGE 	    },
	{ "spit",       "spits",    DAMAGE_SPIT 	    },
	{ "sting",      "stings",   DAMAGE_WTALON 	    },
	{ "posion",     "poisons",  DAMAGE_POISON 	    }
};

const struct plane_types plane_table[]=
{
	{ "dead plane",		PLANE_DEAD      },	
	{ "normal plane",	PLANE_NORMAL },
	{ "shadow plane",	PLANE_SHADOW },
	{ "spirit plane",	PLANE_SPIRIT },
	{ "umbra plane",		PLANE_UMBRA      },	
	{ "ethereal plane",	PLANE_ETHEREAL  },	
	{ "immortal plane",	PLANE_IMMORTAL  },	
	{ "earth plane",		PLANE_EARTH  },	
	{ "punishment plane",	PLANE_PUNISHMENT  },	
	{ "", 0  },	
};


const struct cset_powers_type cset_powers_list[]=
{
	{ "abombwe",	DISC_ABOMBWE, CLS_VAMP },
	{ "animalism",	DISC_ANIMALISM, CLS_VAMP },
	{ "auspex",		DISC_AUSPEX, CLS_VAMP },
	{ "celerity",	DISC_CELERITY, CLS_VAMP },
	{ "chimerstry", DISC_CHIMERSTRY, CLS_VAMP },
	{ "daimoinon",	DISC_DAIMOINON, CLS_VAMP },
	{ "deimos",		DISC_DEIMOS, CLS_VAMP },
	{ "dementation", DISC_DEMENTATION, CLS_VAMP },
	{ "dominate",	DISC_DOMINATE, CLS_VAMP },
	{ "fortitude",	DISC_FORTITUDE, CLS_VAMP },
	{ "grotsquous", DISC_GROTESQUOUS, CLS_VAMP },
	{ "kai",		DISC_KAI, CLS_VAMP },
	{ "melpominee", DISC_MELPOMINEE, CLS_VAMP },
	{ "mortis",		DISC_MORTIS, CLS_VAMP },
	{ "necromancy", DISC_NECROMANCY, CLS_VAMP },
	{ "nihilistics", DISC_NIHILISTICS, CLS_VAMP },
	{ "obeah",		DISC_OBEAH, CLS_VAMP },
	{ "obfuscate",	DISC_OBFUSCATE, CLS_VAMP },
	{ "obtenebration",DISC_OBTENEBRATION, CLS_VAMP },
	{ "ogham",		DISC_OGHAM, CLS_VAMP },
	{ "potence",	DISC_POTENCE, CLS_VAMP },
	{ "presence",	DISC_PRESENCE, CLS_VAMP },
	{ "protean",	DISC_PROTEAN, CLS_VAMP },
	{ "quietus",	DISC_QUIETUS, CLS_VAMP },
	{ "rift",		DISC_RIFT, CLS_VAMP },
	{ "sanguinus",	DISC_SANGUINUS, CLS_VAMP },
	{ "serpentis",	DISC_SERPENTIS, CLS_VAMP },
	{ "temporis",	DISC_TEMPORIS, CLS_VAMP },
	{ "thanatosis",	DISC_THANATOSIS, CLS_VAMP },
	{ "thaumaturgy", DISC_THAUMATURGY, CLS_VAMP },
	{ "visceratika", DISC_VISCERATIKA, CLS_VAMP },
	{ "homid",	BREED_HOMID, CLS_BREED },
	{ "lupus",	BREED_LUPUS, CLS_BREED },
	{ "metis",	BREED_METIS, CLS_BREED },
	{ "ragabash",	AUSPICE_RAGABASH,	CLS_AUSPICE },
	{ "theurge",	AUSPICE_THEURGE,	CLS_AUSPICE },
	{ "philodox",	AUSPICE_PHILODOX,	CLS_AUSPICE },
	{ "galliard",	AUSPICE_GALLIARD,	CLS_AUSPICE },
	{ "ahroun",		AUSPICE_AHROUN,		CLS_AUSPICE },
	{ "black_furies",TRIBE_BLACK_FURIES,CLS_TRIBE },
	{ "bone_gnawers",TRIBE_BONE_GNAWERS,CLS_TRIBE },
	{ "fianna",		TRIBE_FIANNA,		CLS_TRIBE },
	{ "get_of_fenris",	TRIBE_GET_OF_FENRIS, CLS_TRIBE },
	{ "glass_walkers",	TRIBE_GLASS_WALKERS, CLS_TRIBE },
	{ "red_talons",	TRIBE_RED_TALONS,	CLS_TRIBE },
	{ "shadow_lords", TRIBE_SHADOW_LORDS,	CLS_TRIBE },
	{ "silent_striders",	TRIBE_SILENT_STRIDERS, CLS_TRIBE },
	{ "silver_fangs",	TRIBE_SILVER_FANGS, CLS_TRIBE },
	{ "stargazers",	TRIBE_STARGAZERS, CLS_TRIBE },
	{ "uktena",		TRIBE_UKTENA,	CLS_TRIBE },
	{ "wendigo",	TRIBE_WENDIGO,	CLS_TRIBE },
	{ "child_of_spiral", TRIBE_CHILD_OF_SPIRAL, CLS_TRIBE },
	{ "correspondence",	SPHERE_CORRESPONDENCE, CLS_MAGE },
	{ "entropy",	SPHERE_ENTROPY, CLS_MAGE },
	{ "forces",		SPHERE_FORCES,	CLS_MAGE },
	{ "mind",		SPHERE_MIND,	CLS_MAGE },
	{ "matter",		SPHERE_MATTER,	CLS_MAGE },
	{ "prime",		SPHERE_PRIME,	CLS_MAGE },
	{ "life",		SPHERE_LIFE,	CLS_MAGE },
	{ "time",		SPHERE_TIME,	CLS_MAGE },
	{ NULL,	0, 0 }

};

const struct ghostaffect_type ghostaffect_table[]=
{
	{"sanctuary", AFF_SANCTUARY, GHOST_SANCTUARY},
	{"iceshield", ITEMA_ICESHIELD, GHOST_ICESHIELD},
	{"fireshield", ITEMA_FIRESHIELD, GHOST_FIRESHIELD},
	{"shockshield", ITEMA_SHOCKSHIELD, GHOST_SHOCKSHIELD},
	{"acidshield", ITEMA_ACIDSHIELD, GHOST_ACIDSHIELD},
	{"protect", AFF_PROTECT, GHOST_PROTECT},
	{"flying", AFF_FLYING, GHOST_FLYING},
	{"invisible", AFF_INVISIBLE, GHOST_INVISIBLE},
	{"sneak", AFF_SNEAK, GHOST_SNEAK},
	{NULL,0,0}
};
